<?php
/**
 * classes/ibit_Evaluation.class.php
 *
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  Ivn Marcos <ivan.marcos@gmx.net>
 * @author  Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package IBIT_Evaluation
 * @copyright Copyright 2006, 2007 IBIT
 * @version 1.2
 */

/**
 * Die Reprsentation einer Evaluation.
 *
 * Diese Klasse stellt eine Reprsentation der Daten einer Evaluation dar. Die Daten einer Evaluation knnen hier geladen, gespeichert oder
 * verndert werden. Ebenso werden statische Funktionen zum Auslesen verschiedener Evaluationen angeboten.
 *
 * @author     Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author     Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @package    IBIT_Evaluation
 * @subpackage Structure
 * @copyright  Copyright 2006, 2007 IBIT
 * @version    1.7
 */
class ibit_Evaluation
{
	/**
	 * Datenbankverbindungsobjekt
	 * @var object $conn
	 */
	var $conn = null;

	/**
	 * SQL-Statement zum Laden einer Evaluation
	 * @var string $sql_load_evaluation
	 */
	var $sql_load_evaluation = 'SELECT e.evaluation_template_id, seminar_id, institut_id, fakultaet_id, semester_id, UNIX_TIMESTAMP(start_date) AS start_date, UNIX_TIMESTAMP(stop_date) AS stop_date, UNIX_TIMESTAMP(create_date) AS create_date, create_author_id, UNIX_TIMESTAMP(change_date) AS change_date, change_author_id, status, confirmed, visible_date, visible_author_id, comment, SUM(u.hidden<>1) AS participants FROM ibit_evaluation AS e LEFT JOIN ibit_evaluation_user AS u ON (e.evaluation_id=u.evaluation_id) WHERE e.evaluation_id = ? GROUP BY e.evaluation_id';

  /**
   * SQL-Statement zum ndern einer Evaluation
   * @var string $sql_update_evaluation
   */
  var $sql_update_evaluation = 'UPDATE ibit_evaluation SET evaluation_template_id=?, seminar_id=?, institut_id=?, fakultaet_id=?, semester_id=?, start_date=FROM_UNIXTIME(?), stop_date=FROM_UNIXTIME(?), change_date=NOW(), change_author_id=?, `status`=?, confirmed=?, visible_date=FROM_UNIXTIME(?), visible_author_id=?, comment=? WHERE evaluation_id = ?';

  /**
   * SQL-Statement zum Einfgen einer Evaluation
   * @var string $sql_insert_evaluation
   */
  var $sql_insert_evaluation = 'INSERT INTO ibit_evaluation (evaluation_template_id, seminar_id, institut_id, fakultaet_id, semester_id, start_date, stop_date, change_date, change_author_id, `status`, confirmed, visible_date, visible_author_id, comment, create_date, create_author_id) VALUES (?, ?, ?, ?, ?, FROM_UNIXTIME(?), FROM_UNIXTIME(?), NOW(), ?, ?, ?, FROM_UNIXTIME(?), ?, ?, NOW(), ?)';

	/**
	 * Die ID der Evaluation
	 * @var string $evaluation_id
	 */
	var $evaluation_id = null;

	/**
	 * Die Template-ID der Evaluation
	 * @var string $template_id
	 */
	var $template_id;

	/**
	 * Das Template-Objekt der Evaluation
	 * @var object $template
	 */
	var $template;

	/**
	 * ID des zur Evaluation gehrenden Seminars
	 * @var string $seminar_id
	 */
	var $seminar_id;

	/**
	 * ID des zur Evaluation gehrenden Instituts
	 * @var string $institute_id
	 */
	var $institute_id;

	/**
	 * ID der zur Evaluation gehrenden Fakultt
	 * @var string $faculty_id
	 */
	var $faculty_id;

	/**
	 * ID des zur Evaluation gehrenden Semesters
	 * @var string $semester_id
	 */
	var $semester_id;

	/**
	 * Objekt des zur Evaluation gehrenden Seminars
	 * @var mixed $seminar
	 */
	var $seminar;

	/**
	 * Der Status der Evaluation
	 * @var int $status
	 */
	var $status;

	/**
	 * Gibt an, ob die Evaluation freigegeben wurde
	 * @var boolean $confirmed
	 */
	var $confirmed;

	/**
	 * Das Startdatum der Evaluation
	 * @var int $start_date
	 */
	var $start_date;

	/**
	 * Das Enddatum der Evaluation
	 * @var int $stop_date
	 */
	var $stop_date;

	/**
	 * Das Datum der letzten nderung
	 * @var int $change_date
	 */
	var $change_date;

	/**
	 * Die Id des Autoren der letzten nderung
	 * @var string $change_author_id
	 */
	var $change_author_id;

	/**
	 * Das Erstellungsdatum der Evaluation
	 * @var int $create_date
	 */
	var $create_date;

	/**
	 * Die Id des Autoren, der die Evaluation erstellt hat
	 * @var string $create_author_id
	 */
	var $create_author_id;

	/**
	 * Das Datum, an dem die Evaluation freigeschaltet worden ist oder null
	 * @var int $visible_date
	 */
	var $visible_date;

	/**
	 * Die Id des Users, der die Evaluation freigeschaltet hat oder null
	 * @var string $visible_author
	 */
	var $visible_author_id;

	/**
	 * Der Kommentar der Evaluation
	 * @var string $comment
	 */
	var $comment;

	/**
	 * Die Anzahl der Teilnehmer
	 * @var string $participants
	 */
	var $participants;

	/**
	 * Konstruktor der Klasse.
	 * @param int $evaluation_id Id der Evaluation
	 * @uses ibit_DatabaseConnection
	 */
	function ibit_Evaluation( $evaluation_id = null )
	{
		$this->conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		if ( $evaluation_id !== null )
		{
			$this->load( $evaluation_id );
		}
	}

	/**
	 * Ldt die Daten der Evaluation mit der bergebenen Id in dieses Objekt.
	 * @param string $eval_id Id der Evaluation
	 * @return boolean Statusmeldung
	 */
	function load( $eval_id )
	{
		$this->evaluation_id = $eval_id;
		$loadResult = $this->conn->Execute( $this->sql_load_evaluation, $eval_id );
		if (!$loadResult or $loadResult->EOF)
		{
			return false;
		}
		$this->seminar_id = $loadResult->fields['seminar_id'];
		$this->institute_id = $loadResult->fields['institut_id'];
		$this->faculty_id = $loadResult->fields['fakultaet_id'];
		$this->template_id = $loadResult->fields['evaluation_template_id'];
		$this->semester_id = $loadResult->fields['semester_id'];
		$this->comment = $loadResult->fields['comment'];
		$this->status = $loadResult->fields['status'];
		$this->confirmed = $loadResult->fields['confirmed'];
		$this->start_date = $loadResult->fields['start_date'];
		$this->stop_date = $loadResult->fields['stop_date'];
		$this->create_date = $loadResult->fields['create_date'];
		$this->create_author_id = $loadResult->fields['create_author_id'];
		$this->change_date = $loadResult->fields['change_date'];
		$this->change_author_id = $loadResult->fields['change_author_id'];
		$this->visible_date = $loadResult->fields['visible_date'];
		$this->visible_author_id = $loadResult->fields['visible_author_id'];
		$this->participants = $loadResult->fields['participants'];
		$loadResult->Close();

		return true;
	}

	/**
     * Speichert die Daten der Evaluation in der Datenbank.
     */
	function save()
	{
		$data = Array();
		array_push( $data, $this->template_id );
		array_push( $data, $this->seminar_id );
		array_push( $data, $this->institute_id );
		array_push( $data, $this->faculty_id );
		array_push( $data, $this->semester_id );
		array_push( $data, $this->start_date );
		array_push( $data, $this->stop_date );
		array_push( $data, $this->change_author_id );
		array_push( $data, $this->status );
		array_push( $data, $this->confirmed );
		array_push( $data, $this->visible_date );
		array_push( $data, $this->visible_author_id );
		array_push( $data, $this->comment );
		if ( $this->evaluation_id === null)
		{
			array_push( $data, $this->create_author_id );
			$this->conn->Execute( $this->sql_insert_evaluation, $data );
			$this->evaluation_id = $this->conn->Insert_Id();
		}
		else
		{
			array_push($data, $this->evaluation_id);
			$this->conn->Execute( $this->sql_update_evaluation, $data );
		}
	}

	/**
	 * Lscht eine Evaluation oder eine Menge von Evaluationen und smtliche Querverweise aus der Datenbank.
	 * @param mixed $evaluation_ids Id(s) der Evaluation(en)
	 * @return boolean Statusmeldung
	 * @uses ibit_DatabaseConnection
	 */
	function delete( $evaluation_ids )
	{
		$conn = isset( $this->conn ) ? $this->conn : new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		if ( empty( $evaluation_ids ) )
		{
			return false;
		}

		if ( !is_array( $evaluation_ids ) )
		{
			$evaluation_ids = Array( $evaluation_ids );
		}

		$evaluation_string = implode( ',', $evaluation_ids );
		$sql_delete_evaluation = 'DELETE QUICK FROM ibit_evaluation WHERE evaluation_id IN ('.$evaluation_string.')';
		$sql_delete_evaluation_answers = 'DELETE QUICK FROM ibit_evaluation_answers WHERE evaluation_id IN ('.$evaluation_string.')';
		$sql_delete_evaluation_users = 'DELETE QUICK FROM ibit_evaluation_user WHERE evaluation_id IN ('.$evaluation_string.')';
		$sql_delete_evaluation_summary_views = 'DELETE QUICK FROM ibit_evaluation_summary_views WHERE evaluation_id IN ('.$evaluation_string.')';
		$sql_delete_optimize_after = 'OPTIMIZE TABLE ibit_evaluation, ibit_evaluation_answers, ibit_evaluation_user, ibit_evaluation_summary_views';

		$conn->Execute( $sql_delete_evaluation );
		$deleted_evaluations = $conn->Affected_Rows();
		$conn->Execute( $sql_delete_evaluation_answers );
		$conn->Execute( $sql_delete_evaluation_users );
		$conn->Execute( $sql_delete_evaluation_summary_views );
		$conn->Execute( $sql_delete_optimize_after );

		return $deleted_evaluations > 0;
	}

	/**
	 * Liefert die Id der Evaluation zurck.
	 * @return string Id der Evaluation
	 */
	function getEvaluationId()
	{
		return $this->evaluation_id;
	}

	/**
	 * Liefert die Id der Evaluation zurck.
	 * @return string Id der Evaluation
	 */
	function getId()
	{
		return $this->evaluation_id;
	}

	/**
	 * Liefert die Vorlagen-Id der Evaluation zurck.
	 * @return string Id der Vorlage
	 */
	function getTemplateId()
	{
		return $this->template_id;
	}

	/**
	 * Setzt die Vorlagen-Id der Evaluation.
	 * @param $template_id Id der Vorlage
	 */
	function setTemplateId( $template_id )
	{
		$this->template_id = $template_id;
	}

	/**
	 * Ldt die zur Evaluation gehrende Vorlage
	 * @return boolean Statusmeldung
	 * @uses ibit_EvaluationTemplate
	 */
	function loadTemplate()
	{
		if ( $this->template_id === null )
		{
			return false;
		}

		$this->template = new ibit_EvaluationTemplate( $this->template_id );
		return true;
	}

	/**
	 * Liefert die Id des zur Evaluation gehrenden Seminars zurck.
	 * @return string Id des Seminars
	 */
	function getSeminarId()
	{
		return $this->seminar_id;
	}

	/**
	 * Setzt die Id des zur Evaluation gehrenden Seminars.
	 * @param string $seminar_id Id des Seminars
	 */
	function setSeminarId( $seminar_id )
	{
		$this->seminar_id = $seminar_id;
	}

	/**
	 * Ldt das zur Evaluation gehrende Seminar.
	 * @return boolean Statusmeldung
	 * @uses ibit_EvaluationSeminar
	 */
	function loadSeminar()
	{
		if ( $this->seminar_id === null)
		{
			return false;
		}

		$this->seminar = new ibit_EvaluationSeminar( $this->seminar_id );
		return true;
	}

	/**
	 * Liefert die Id des zur Evaluation gehrenden Instituts zurck.
	 * @return string Id des Instituts
	 */
	function getInstitutId()
	{
		return $this->institute_id;
	}

	/**
	 * Setzt die Id des zur Evaluation gehrenden Instituts.
	 * @param string $institute_id Id des Instituts
	 */
	function setInstitutId( $institute_id )
	{
		$this->institute_id = $institute_id;
	}

	/**
	 * Liefert die Id der zur Evaluation gehrenden Fakultt zurck.
	 * @return string Id der Fakultt
	 */
	function getFakultaetId()
	{
		return $this->faculty_id;
	}

	/**
	 * Setzt die Id der zur Evaluation gehrenden Fakultt.
	 * @param string $faculty_id Id der Fakultt
	 */
	function setFakultaetId( $faculty_id )
	{
		$this->faculty_id = $faculty_id;
	}

	/**
	 * Liefert die Id des zur Evaluation gehrenden Semesters zurck.
	 * @return string Id des Semesters
	 */
	function getSemesterId()
	{
		return $this->semester_id;
	}

	/**
	 * Setzt die Id des zur Evaluation gehrenden Semesters.
	 * @param string $semester_id Id des Semesters
	 */
	function setSemesterId( $semester_id )
	{
		$this->semester_id = $semester_id;
	}

	/**
	 * Liefert den Status der Evaluation zurck.
	 * @return int Status der Evaluation
	 */
	function getStatus()
	{
		return $this->status;
	}

	/**
	 * Setzt den Status der Evaluation.
	 * @param int $status
	 */
	function setStatus( $status )
	{
		$this->status = $status;
	}

	/**
	 * Liefert den Besttigt-Status der Evaluation zurck.
	 * @return string Besttigt-Status der Evaluation
	 */
	function getConfirmed()
	{
		return $this->confirmed;
	}

	/**
	 * Setzt den Besttigt-Status der Evaluation.
	 * @param string $confirmed Neuer Besttigt-Status
	 */
	function setConfirmed( $confirmed )
	{
		$this->confirmed = $confirmed;
	}

	/**
	 * Setzt den Kommentar der Evaluation.
	 * @param string $comment
	 */
	function setComment( $comment )
	{
		$this->comment = $comment;
	}

	/**
	 * Liefert den Kommentar der Evaluation zurck.
	 * @return string Kommentar der Evaluation
	 */
	function getComment()
	{
		return $this->comment;
	}

	/**
	 * Liefert das Startdatum der Evaluation zurck.
	 * @return int Startdatum der Evaluation als Unix-Timestamp.
	 */
	function getStartDate()
	{
		return $this->start_date;
	}

	/**
	 * Setzt das Startdatum der Evaluation
	 * @param mixed $start_date Startdatum der Evaluation als Unix-Timestamp
	 */
	function setStartDate( $start_date = null )
	{
		if ( $start_date === null )
		{
			$start_date = time();
		}
		elseif ( !is_int( $start_date ) )
		{
			$start_date = strtotime( $start_date );
		}
		$this->start_date = $start_date;
	}

	/**
	 * Liefert das Enddatum der Evaluation zurck.
	 * @return int Enddatum der Evaluation als Unix-Timestamp
	 */
	function getStopDate()
	{
		return $this->stop_date;
	}

	/**
	 * Setzt das Enddatum der Evaluation.
	 * @param mixed $stop_date Enddatum der Evaluation als Unix-Timestamp.
	 */
	function setStopDate( $stop_date = null )
	{
		if ( $stop_date === null )
		{
			$stop_date = time();
		}
		elseif ( !is_int( $stop_date ) )
		{
			$stop_date = strtotime( $stop_date );
		}
		$this->stop_date = $stop_date;
	}

	/**
	 * Liefert das nderungsdatum der Evaluation zurck.
	 * @return int nderungsdatum der Evaluation als Unix-Timestamp
	 */
	function getChangeDate()
	{
		return $this->change_date;
	}

	/**
	 * Liefert die Autoren-Id der letzten nderung zurck.
	 * @return string Id des Autoren
	 */
	function getChangeAuthorId()
	{
		return $this->change_author_id;
	}

	/**
	 * Setzt die Autoren-Id der letzten nderung.
	 * @param String $author_id Id des Autoren.
	 */
	function setChangeAuthorId( $change_author_id )
	{
		$this->change_author_id = $change_author_id;
	}

	/**
	 * Liefert das Erstellungsdatum der Evaluation zurck.
	 * @return int Erstellungsdatum der Evaluation als Unix-Timestamp
	 */
	function getCreateDate()
	{
		return $this->create_date;
	}

	/**
	 * Liefert die Autoren-Id des Users zurck, der die Evaluation erstellt hat.
	 * @return string Id des Autors
	 */
	function getCreateAuthorId()
	{
		return $this->create_author_id;
	}

	/**
	 * Setzt die Autoren-Id der Evaluation.
	 * @param String $author_id Id des Autors
	 */
	function setCreateAuthorId( $create_author_id )
	{
		$this->create_author_id = $create_author_id;
	}

	/**
	 * Liefert zurck, ob die Evaluation freigeschaltet ist oder nicht.
	 * @return boolean Evaluation freigeschaltet?
	 */
	function isVisible()
	{
		return $this->visible_date !== null;
	}

	/**
	 * Liefert die Id des Users zurck, der die Evaluation freigeschaltet hat.
	 * @return string Id des Users
	 */
	function getVisibleAuthorId()
	{
		return $this->visible_author_id;
	}

	/**
	 * Liefert das Datum zurck, an dem die Evaluation freigeschaltet wurde.
	 * @return int Freischalatungsdatum der Evaluation
	 */
	function getVisibleDate()
	{
		return $this->visible_date;
	}

	/**
	 * Schaltet eine Evaluation frei oder nimmt diese Freischaltung zurck.
	 * @param boolean $state Freigeschaltet?
	 * @param string $author Id des Users, der die nderung vornimmt
	 */
	function setVisible( $state, $visible_author_id )
	{
		if ($state)
		{
			$this->visible_date = time();
			$this->visible_author_id = $visible_author_id;
		}
		else
		{
			$this->visible_date = null;
			$this->visible_author_id = null;
		}
	}

	/**
	 * Liefert die Anzahl der Teilnehmer zurck.
	 * @return int Anzahl der Teilnehmer
	 */
	function getNumberOfParticipants()
	{
		return $this->participants;
	}

	/**
	 * Bricht das Objekt in ein mehrdimensionales Array auf, das alle relevanten Informationen enthlt.
	 * Das Array hat dabei folgenden Aufbau:
	 *
	 * <pre>
	 *   [
	 *     evaluation_id    => Id der Evaluation,
	 *     template_id      => Id der Vorlage,
	 *     seminar_id       => Id des Seminars,
	 *     institut_id      => Id des Instituts,
	 *     fakultaet_id     => Id der Fakultaet,
	 *     status           => Status der Evaluation,
	 *     confirm          => Besttigt-Status der Evaluation (nicht benutzt),
	 *     comment          => Kommentar der Evaluation,
	 *     semester_id      => Id des Semesters,
	 *     start_date       => Startdatum der Evaluation,
	 *     stop_date        => Enddatum der Evaluation,
	 *     change_data      => nderungsdatum der Evaluation,
	 *     change_author_id => Id des Users, der die letzte nderung durchgefhrt hat,
	 *     create_data      => Erstelldatum der Evaluation,
	 *     create_author_id => Id des Users, der die Evaluation erstellt hat,
	 *     visible_data     => Freischaltdatum der Evaluation,
	 *     visible_author   => Id des Users, der die Evaluation freigeschaltet hat,
	 *     participants     => Anzahl der User, die an der Evaluation teilgenommen haben,
	 *     template         => Vorlagendaten [
	 *                           ...
	 *                                       ],
	 *     seminar          => Seminardaten [
	 *                           ...
	 *                                      ]
	 *    ]
	 * </pre>
	 *
	 * @return array Das Objekt als Array
	 * @uses ibit_EvaluationTemplate::flattenToArray()
	 * @uses ibit_EvaluationSeminar::flattenToArray();
	 */
	function flattenToArray()
	{
		$result = Array();

		$result['evaluation_id'] = $this->evaluation_id;
		$result['template_id'] = $this->template_id;
		$result['seminar_id'] = $this->seminar_id;
		$result['institut_id'] = $this->institute_id;
		$result->fields['fakultaet_id'] = $this->faculty_id;
		$result['status'] = $this->status;
		$result['confirmed'] = $this->confirmed;
		$result['comment'] = $this->comment;
		$result['semester_id'] = $this->semester_id;
		$result['start_date'] = $this->start_date;
		$result['stop_date'] = $this->stop_date;
		$result['change_date'] = $this->change_date;
		$result['change_author_id'] = $this->change_author_id;
		$result['create_date'] = $this->create_date;
		$result['create_author_id'] = $this->create_author_id;
		$result['visible_date'] = $this->visible_date;
		$result['visible_author'] = $this->visible_author_id;
		$result['participants'] = $this->participants;

		if ( $this->template === null )
		{
			$this->loadTemplate();
		}
		$result['template'] = $this->template->flattenToArray();

		if ( $this->seminar === null )
		{
			$this->loadSeminar();
		}
		$result['seminar'] = $this->seminar->flattenToArray();

		return $result;
	}

########################
# STATISCHE FUNKTIONEN #
########################

	/**
	 * Ldt alle relevanten Evaluationsdaten von abgelaufenen Evaluationen eines Semesters gruppiert nach Fakultten.
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsFacultyOverview()
	 */
	function getExpiredEvaluationsFacultyOverview( $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsFacultyOverview( $semester_id, 'stop_date < NOW()' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von laufenden Evaluationen eines Semesters gruppiert nach Fakultten.
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsFacultyOverview()
	 */
	function getCurrentEvaluationsFacultyOverview( $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsFacultyOverview( $semester_id, 'NOW() BETWEEN start_date AND stop_date' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von zuknftigen Evaluationen eines Semesters gruppiert nach Fakultten.
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsFacultyOverview()
	 */
	function getFutureEvaluationsFacultyOverview( $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsFacultyOverview( $semester_id, 'start_date > NOW()' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten eines Semesters gruppiert nach Fakultten.
	 * Die Rckgabe wird dabei nach Fakulttsnamen sortiert ausgegeben und hat folgenden Aufbau:
	 *
	 * <pre>
	 *    [
	 *     fakultaet_id     => Id der Fakultt,
	 *     fakultaet_name   => Name der Fakultt,
	 *     seminar_count    => Summe der Seminare,
	 *     institute_count  => Summe der Institute,
	 *     evaluation_count => Summe der Evaluationen,
	 *     participants     => Summe der Teilnehmer an den Evaluationen,
	 *     seminar_members  => Summe der Teilnehmer an den Seminaren
	 *    ]{0..x}
	 * </pre>
	 *
	 * @param string $semester_id Id des Semesters
	 * @param string $additional_where_clause Optionale Einschrnkungen des Queries
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_DatabaseConnection
	 * @uses ibit_Helper::getFacultyName()
	 * @uses ibit_EvaluationSeminar::getSeminarMemberCountByFacultyId()
	 */
	function getEvaluationsFacultyOverview( $semester_id = -1, $additional_where_clause = '' )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = 'SELECT fakultaet_id, COUNT(DISTINCT seminar_id) AS seminar_count, COUNT(DISTINCT institut_id) AS institute_count, COUNT(DISTINCT e.evaluation_id) AS evaluation_count, SUM(u.hidden<>1) AS participants FROM ibit_evaluation e LEFT JOIN ibit_evaluation_user AS u USING(evaluation_id) ###WHERECLAUSE### GROUP BY e.fakultaet_id';
		$where_clause = '';
		if ( $semester_id != -1 )
		{
			if ( !empty( $additional_where_clause ) )
			{
				$where_clause = $additional_where_clause . ' AND ';
			}
			$where_clause .= "semester_id = '{$semester_id}'";
		}

		if ( !empty( $where_clause ) )
		{
			$where_clause = 'WHERE '.$where_clause;
		}
		$sql = str_replace( '###WHERECLAUSE###', $where_clause, $sql );

		$faculty_result = $conn->Execute( $sql );

		if ( !$faculty_result )
		{
			return false;
		}

		$result = Array();

		while ( !$faculty_result->EOF )
		{
			$item = Array();
			$item['fakultaet_id'] = $faculty_result->fields['fakultaet_id'];
			$item['fakultaet_name'] = ibit_Helper::getFacultyName( $item['fakultaet_id'] );
			$item['seminar_count'] = $faculty_result->fields['seminar_count'];
			$item['institute_count'] = $faculty_result->fields['institute_count'];
			$item['evaluation_count'] = $faculty_result->fields['evaluation_count'];
			$item['participants'] = $faculty_result->fields['participants'];
			$item['seminar_members'] = ibit_EvaluationSeminar::getSeminarMemberCountByFacultyId( $item['fakultaet_id'], $semester_id, $additional_where_clause );

			array_push( $result, $item );

			$faculty_result->MoveNext();
		}
		$faculty_result->Close();

		# Ausgabe sortieren nach Fakulttsnamen
		if ( !function_exists('faculty_compare') )
		{
			function faculty_compare( $a, $b )
			{
				return strcmp( $a['fakultaet_name'], $b['fakultaet_name'] );
			}
		}
		usort( $result, 'faculty_compare' );

		return $result;
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von abgelaufenen Evaluationen einer Fakultt und eines Semesters gruppiert nach Instituten.
	 * @param string $faculty_id Id der Fakultt
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsByFacultyId()
	 */
	function getExpiredEvaluationsByFacultyId( $faculty_id, $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsByFacultyId( $faculty_id, $semester_id, 'stop_date < NOW()' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von laufenden Evaluationen einer Fakultt und eines Semesters gruppiert nach Instituten.
	 * @param string $faculty_id Id der Fakultt
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsByFacultyId()
	 */
	function getCurrentEvaluationsByFacultyId( $faculty_id, $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsByFacultyId( $faculty_id, $semester_id, 'NOW() BETWEEN start_date AND stop_date' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von zuknftigen Evaluationen einer Fakultt und eines Semesters gruppiert nach Instituten.
	 * @param string $faculty_id Id der Fakultt
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsByFacultyId()
	 */
	function getFutureEvaluationsByFacultyId( $faculty_id, $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsByFacultyId( $faculty_id, $semester_id, 'start_date > NOW()' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten einer Fakultt und eines Semesters gruppiert nach Instituten.
	 * Die Rckgabe hat dabei folgenden Aufbau:
	 *
	 * <pre>
	 *    [
	 *     institut_id      => Id des Instituts,
	 *     institut_name    => Name des Instituts,
	 *     seminar_count    => Summe der Seminare,
	 *     evaluation_count => Summe der Evaluationen,
	 *     participants     => Summe der Teilnehmer an den Evaluationen,
	 *     seminar_members  => Summe der Teilnehmer an den Seminaren
	 *    ]{0..x}
	 * </pre>
	 *
	 * @param string $faculty_id Id der Fakultt
	 * @param string $semester_id Id des Semesters
	 * @param string $additional_where_clause Optionale Einschrnkungen des Queries
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_DatabaseConnection
	 * @uses ibit_EvaluationSeminar::getSeminarMemberCountByInstituteId()
	 */
	function getEvaluationsByFacultyId( $faculty_id, $semester_id = -1, $additional_where_clause = '' )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = 'SELECT e.institut_id, i.name AS institut_name, COUNT(DISTINCT seminar_id) AS seminar_count, COUNT(DISTINCT e.evaluation_id) AS evaluation_count, SUM(u.hidden<>1) AS participants FROM ibit_evaluation AS e INNER JOIN Institute AS i ON (BINARY e.institut_id=BINARY i.Institut_id) LEFT JOIN ibit_evaluation_user AS u ON (e.evaluation_id = u.evaluation_id) WHERE fakultaet_id = ? ###WHERECLAUSE### GROUP BY institut_id ORDER BY institut_name';

		$where_clause = '';
		if ( $semester_id != -1 )
		{
			if ( !empty( $additional_where_clause ) )
			{
				$where_clause = $additional_where_clause . ' AND ';
			}
			$where_clause .= "semester_id = '{$semester_id}'";
		}

		if ( !empty( $where_clause ) )
		{
			$where_clause = 'AND '.$where_clause;
		}

		$sql = str_replace( '###WHERECLAUSE###', $where_clause, $sql );

		$institute_result = $conn->Execute( $sql, $faculty_id );

		if ( !$institute_result )
		{
			return false;
		}

		$result = Array();

		while ( !$institute_result->EOF )
		{
			$item = Array();
			$item['institut_id'] = $institute_result->fields['institut_id'];
			$item['institut_name'] = $institute_result->fields['institut_name'];
			$item['seminar_count'] = $institute_result->fields['seminar_count'];
			$item['evaluation_count'] = $institute_result->fields['evaluation_count'];
			$item['participants'] = $institute_result->fields['participants'];
			$item['seminar_members'] = ibit_EvaluationSeminar::getSeminarMemberCountByInstituteId( $item['institut_id'], $semester_id, $additional_where_clause );

			array_push( $result, $item );

			$institute_result->MoveNext();
		}
		$institute_result->Close();

		return $result;
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von abgelaufenen Evaluationen eines Instituts und eines Semesters.
	 * @param string $institute_ids Ids der Institute
	 * @param int $skip Anzahl der Evaluationen, die bersprungen werden sollen
	 * @param int $limit Anzahl der Evaluationen, die ausgelesen werden sollen
	 * @param int $count Anzahl der ausgelesenen Evaluationen (Output-Parameter)
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsByInstituteId()
	 */
	function getExpiredEvaluationsByInstituteId( $institute_ids, &$skip, &$limit, &$count, $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsByInstituteId( $institute_ids, $skip, $limit, $count, $semester_id, 'stop_date < NOW()' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von laufenden Evaluationen eines Instituts und eines Semesters.
	 * @param string $institute_ids Ids der Institute
	 * @param int $skip Anzahl der Evaluationen, die bersprungen werden sollen
	 * @param int $limit Anzahl der Evaluationen, die ausgelesen werden sollen
	 * @param int $count Anzahl der ausgelesenen Evaluationen (Output-Parameter)
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsByInstituteId()
	 */
	function getCurrentEvaluationsByInstituteId( $institute_ids, &$skip, &$limit, &$count, $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsByInstituteId( $institute_ids, $skip, $limit, $count, $semester_id, 'NOW() BETWEEN start_date AND stop_date' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten von zuknftigen Evaluationen eines Instituts und eines Semesters.
	 * @param string $institute_ids Ids der Institute
	 * @param int $skip Anzahl der Evaluationen, die bersprungen werden sollen
	 * @param int $limit Anzahl der Evaluationen, die ausgelesen werden sollen
	 * @param int $count Anzahl der ausgelesenen Evaluationen (Output-Parameter)
	 * @param string $semester_id Id des Semesters
	 * @return array Evaluationsdaten
	 * @static
	 * @uses ibit_Evaluation::getEvaluationsByInstituteId()
	 */
	function getFutureEvaluationsByInstituteId( $institute_ids, &$skip, &$limit, &$count, $semester_id = -1 )
	{
		return ibit_Evaluation::getEvaluationsByInstituteId( $institute_ids, $skip, $limit, $count, $semester_id, 'start_date > NOW()' );
	}

	/**
	 * Ldt alle relevanten Evaluationsdaten eines Instituts und eines Semesters.
	 * Die Rckgabe hat dabei folgenden Aufbau:
	 *
	 * <pre>
	 *    [
	 *     evaluation_id    => Id der Evaluation,
	 *     evaluation_title => Name der Evaluation,
	 *     seminar_id       => Id des Seminars,
	 *     seminar_title    => Name des Seminars
	 *     seminar_members  => Summe der Teilnehmer an dem Seminar,
	 *     template_id      => Id der Vorlage der Evaluation,
	 *     status           => Status der Evaluation,
	 *     start_date       => Startdatum der Evaluation,
	 *     stop_date        => Stopdatum der Evaluation,
	 *     participants     => Anzahl der Teilnehmer an der Evaluation
	 *    ]{0..x}
	 * </pre>
	 *
	 * @param string $institute_ids Ids der Institute
	 * @param int $skip Anzahl der Evaluationen, die bersprungen werden sollen
	 * @param int $limit Anzahl der Evaluationen, die ausgelesen werden sollen
	 * @param int $count Anzahl der ausgelesenen Evaluationen (Output-Parameter)
	 * @param string $semester_id Id des Semesters
	 * @param string $additional_where_clause Optionale Einschrnkungen des Queries
	 * @return array
	 * @static
	 * @uses ibit_DatabaseConnection
	 * @uses ibit_EvaluationSeminar
	 * @uses ibit_Helper::getSeminarName()
	 */
	function getEvaluationsByInstituteId( $institute_ids, &$skip, &$limit, &$count, $semester_id, $additional_where_clause )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql_count = 'SELECT COUNT(*) AS evaluation_count FROM ibit_evaluation WHERE institut_id IN (###IN_CLAUSE###) ###WHERECLAUSE###';

		$sql = "SELECT e.seminar_id, e.evaluation_template_id AS template_id, e.evaluation_id, title AS evaluation_title, UNIX_TIMESTAMP( start_date) AS start_date, UNIX_TIMESTAMP( stop_date ) AS stop_date, CASE WHEN NOW() > stop_date THEN 'expired' WHEN NOW() < start_date THEN 'future' ELSE 'current' END AS status, SUM(u.hidden<>1) AS participants FROM ibit_evaluation AS e INNER JOIN ibit_evaluation_templates AS t USING (evaluation_template_id) LEFT JOIN ibit_evaluation_user AS u ON (u.evaluation_id=e.evaluation_id) WHERE e.institut_id IN (###IN_CLAUSE###) ###WHERECLAUSE### GROUP BY evaluation_id ORDER BY e.evaluation_template_id, e.seminar_id, start_date, stop_date";

		if ( !is_array( $institute_ids ) )
		{
			$institute_ids = array( $institute_ids );
		}

		if ( $skip !== null and $limit !== null )
		{
			$sql .= " LIMIT {$skip}, {$limit}";
		}

		if ( $semester_id != -1 )
		{
			if ( !empty( $additional_where_clause ) )
			{
				$additional_where_clause .= ' AND ';
			}
			$additional_where_clause .= "semester_id = '{$semester_id}'";
		}
		$additional_where_clause = 'AND '.$additional_where_clause;
		$in_clause = "'".implode("','", $institute_ids )."'";

		$sql_count = str_replace( '###IN_CLAUSE###', $in_clause, $sql_count );
		$sql_count = str_replace( '###WHERECLAUSE###', $additional_where_clause, $sql_count );
		$sql = str_replace( '###IN_CLAUSE###', $in_clause, $sql );
		$sql = str_replace( '###WHERECLAUSE###', $additional_where_clause, $sql );

		if ( !($institute_result = $conn->Execute( $sql ) ) )
		{
			return false;
		}

		$result = Array();
		$seminar_ids = Array();
		while ( !$institute_result->EOF )
		{
			$item = Array();
			$item['seminar_id'] = $institute_result->fields['seminar_id'];
			$item['template_id'] = $institute_result->fields['template_id'];
			$item['evaluation_id'] = $institute_result->fields['evaluation_id'];
			$item['evaluation_title'] = $institute_result->fields['evaluation_title'];
			$item['status'] = $institute_result->fields['status'];
			$item['start_date'] = $institute_result->fields['start_date'];
			$item['stop_date'] = $institute_result->fields['stop_date'];
			$item['participants'] = $institute_result->fields['participants'];

			$seminar = new ibit_evaluationSeminar( $item['seminar_id'] );
			$item['seminar_members'] = $seminar->getMembers();

			array_push( $result, $item );
			array_push( $seminar_ids, $item['seminar_id'] );

			$institute_result->MoveNext();
		}
		$institute_result->Close();

		$seminar_names = ibit_Helper::getSeminarName( $seminar_ids );
		foreach ( $result as $idx=>$item )
		{
			$result[$idx]['seminar_title'] = $seminar_names[ $item['seminar_id'] ];
		}

		$count_result = $conn->Execute( $sql_count );
		$count = !$count_result->EOF ? $count_result->fields['evaluation_count'] : 0 ;
		$count_result->Close();

		if ( $skip != null and $limit != null )
		{
			$skip = max ( $skip - $count, 0 );
			$limit = $limit - count( $result );
		}

		return $result;
	}

	/**
	 * Liefert Infos ber die Evaluationen, die einem User zugeordnet sind, zurck.
	 * Es werden nur laufende und beendete Evaluationen ausgelesen. In der Studentensicht werden bei den beendeten
	 * Evaluationen nur die Evaluationen ausgegeben, an denen der Student teilgenommen hat und die laufendenen Evaluationen
	 * werden unterteilt in "teilgenommen" und "nicht teilgenommen". Die Rckgabe wird jeweils nach Status (und
	 * Teilnahmestatus), Semester und Fakultt aufgeteilt ausgegeben.
	 *
	 * Die Rckgabe fr den Dozenten hat folgenden Aufbau:
	 *
	 * <pre>
	 *    [
	 *     running => [
	 *       <semester_name> => [
	 *          <fakultaets_id> => [
	 *             name        => Name der Fakultt,
	 *             evaluations => EVALUATION {0..x}
	 *                             ]{0..x}
	 *                          ]{0..x}
	 *                ]{0..x},
	 *     expired => [
	 *       <semester_name> => [
	 *          <fakultaets_id> => [
	 *             name        => Name der Fakultt,
	 *             evaluations => EVALUATION {0..x}
	 *                             ]{0..x}
	 *                          ]{0..x}
	 *                ]{0..x}
	 *    ]{0..x}
	 * </pre>
	 *
	 * Fr den Studenten hat die Rckgabe folgenden Aufbau:
	 *
	 * <pre>
	 *    [
	 *     running => [
	 *       participated     => [
	 *         <semester_name> => [
	 *            <fakultaets_id> => [
	 *               name        => Name der Fakultt,
	 *               evaluations => EVALUATION {0..x}
	 *                               ]{0..x}
	 *                            ]{0..x}
	 *                           ]{0..x}
	 *                ],
	 *       not_participated => [
	 *         <semester_name> => [
	 *            <fakultaets_id> => [
	 *               name        => Name der Fakultt,
	 *               evaluations => EVALUATION {0..x}
	 *                               ]{0..x}
	 *                            ]{0..x}
	 *                           ]{0..x}
     *                ],
     *     expired => [
	 *       <semester_name> => [
	 *          <fakultaets_id> => [
	 *             name        => Name der Fakultt,
	 *             evaluations => EVALUATION {0..x}
	 *                             ]{0..x}
	 *                          ]{0..x}
	 *                ]{0..x}
	 *    ]{0..x}
	 * </pre>
	 *
	 * EVALUATION wiederum hat dabei folgenden Aufbau:
	 *
	 * <pre>
	 *    [
	 *     evaluation_id   => Id der Evaluation,
	 *     fakultaet_id    => Id der Fakultt,
	 *     semester        => Name des Semesters,
	 *     seminar_id      => Id des Seminars,
	 *     seminar_name    => Name des Seminars,
	 *     seminar_members => Teilnehmer des Seminars,
	 *     participated    => Gibt an, ob der User an der Evaluation teilgenommen hat,
	 *     participants    => Anzahl der Teilnehmer an der Evaluation,
	 *     start_date      => Startdatum der Evaluation,
	 *     stop_date       => Enddatum der Evaluation,
	 *     visible         => Gibt an, ob die Evaluation freigeschaltet ist,
	 *     running         => Gibt an, ob die Evaluation luft oder beendet ist
	 *    ]
	 * </pre>
	 *
	 * @param string $user_id Id des Users
	 * @param boolean $dozent Gibt an, ob der User als Dozent oder Student betrachtet werden soll
	 * @return array Informationen ber die Evaluationen in der Form
	 * @static
	 * @uses ibit_DatabaseConnection
	 * @uses ibit_Helper::getSemester()
	 * @uses ibit_Helper::getFacultyName()
	 * @uses ibit_EvaluationSeminar
	 */
	function getEvaluationDataByUserId( $user_id, $dozent = false )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = "SELECT u.Seminar_id, CONCAT(VeranstaltungsNummer,' - ', Name) AS Name FROM seminar_user AS u INNER JOIN seminare USING(Seminar_id) WHERE user_id = ?";
		if ( $dozent )
		{
			$sql .= " AND u.status IN ('dozent', 'admin')";
		}
		else
		{
			$sql .= " AND u.status NOT IN ('dozent', 'admin')";
		}

		$seminar_result = $conn->Execute( $sql, $user_id );


		if ( !$seminar_result )
		{
			return false;
		}

		$seminar_ids = Array();
		$seminar_names = Array();
		while ( !$seminar_result->EOF )
		{
			$seminar_names[ $seminar_result->fields['Seminar_id'] ] = $seminar_result->fields['Name'];
			array_push( $seminar_ids, $seminar_result->fields['Seminar_id'] );
			$seminar_result->MoveNext();
		}
		$seminar_result->Close();

		if ( empty($seminar_ids) )
		{
			return false;
		}

		$sql = "SELECT e.evaluation_id, seminar_id, semester_id, IF(stop_date>NOW(), 1, 0) AS running, IF(user_id IS NULL, 0, 1) AS participated, UNIX_TIMESTAMP(start_date) AS start_date, UNIX_TIMESTAMP(stop_date) AS stop_date, IF (visible_date IS NULL, 0, 1) AS visible, fakultaet_id FROM ibit_evaluation AS e LEFT JOIN ibit_evaluation_user AS u ON (user_id=? AND u.evaluation_id=e.evaluation_id) WHERE start_date <= NOW() AND seminar_id IN ('".implode("','", $seminar_ids)."') ORDER BY semester_id ASC, stop_date ASC";
		$eval_result = $conn->Execute( $sql, $user_id );

		if ( !$eval_result )
		{
			return false;
		}

		$semester = ibit_Helper::getSemester();
		$semester_names = Array();
		foreach ($semester as $sem)
		{
			$semester_names[ $sem['id'] ] = $sem['name'];
		}

		$temp = Array();
		$eval_ids = Array();
		while ( !$eval_result->EOF )
		{
			$item = Array();
			$item['evaluation_id'] = $eval_result->fields['evaluation_id'];
			$item['semester'] = $semester_names[ $eval_result->fields['semester_id'] ];
			$item['participated'] = $eval_result->fields['participated'];
			$item['seminar_id'] = $eval_result->fields['seminar_id'];
			$item['seminar_name'] = $seminar_names[ $item['seminar_id'] ];
			$item['fakultaet_id'] = $eval_result->fields['fakultaet_id'];
			$item['start_date'] = $eval_result->fields['start_date'];
			$item['stop_date'] = $eval_result->fields['stop_date'];
			$item['visible'] = $eval_result->fields['visible'];
			$item['running'] = $eval_result->fields['running'];

			$seminar = new ibit_EvaluationSeminar( $item['seminar_id'] );
			$item['seminar_members'] = $seminar->getMembers();

			array_push( $temp, $item );
			array_push( $eval_ids, $item['evaluation_id'] );

			$eval_result->MoveNext();
		}
		$eval_result->Close();

		$participants = Array();
		if ( count( $eval_ids ) > 0 )
		{
			$sql_participants = "SELECT evaluation_id, SUM(hidden<>1) AS participants FROM ibit_evaluation_user WHERE evaluation_id IN ('".implode("','", $eval_ids)."') GROUP BY evaluation_id";

			$participants_result = $conn->Execute( $sql_participants );
			while ( !$participants_result->EOF )
			{
				$participants[ $participants_result->fields['evaluation_id'] ] = $participants_result->fields['participants'];
				$participants_result->MoveNext();
			}
			$participants_result->Close();
		}

		if( $dozent )
		{
			$result = Array( 'expired'=>Array(), 'running'=>Array() );
		}
		else
		{
			$result = Array( 'expired'=>Array(), 'running'=>Array('participated'=>Array(), 'not_participated'=>Array()) );
		}

		foreach ( $temp as $item )
		{
			$item['participants'] = empty( $participants[ $item['evaluation_id'] ] ) ? 0 : $participants[ $item['evaluation_id'] ];
			if ( $item['running'] )
			{
				#dozentenansicht
				if( $dozent )
				{
					if ( empty( $result['running'][ $item['semester'] ] ) )
					{
						$result['running'][ $item['semester'] ] = Array();
					}
					if ( empty( $result['running'][ $item['semester'] ][ $item['fakultaet_id'] ] ) )
					{
						$fakultaet_name = ibit_Helper::getFacultyName( $item['fakultaet_id'] );
						$result['running'][ $item['semester'] ][ $item['fakultaet_id'] ] = Array( 'name' => $fakultaet_name, 'evaluations' => Array() );
					}
					array_push( $result['running'][ $item['semester'] ][ $item['fakultaet_id'] ]['evaluations'], $item );
				}
				# studentenansicht
				else
				{
					if ( $item['participated'] )
					{
						if ( empty( $result['running']['participated'][ $item['semester'] ] ) )
						{
							$result['running']['participated'][ $item['semester'] ] = Array();
						}
						if ( empty( $result['running']['participated'][ $item['semester'] ][ $item['fakultaet_id'] ] ) )
						{
							$fakultaet_name = ibit_Helper::getFacultyName( $item['fakultaet_id'] );
							$result['running']['participated'][ $item['semester'] ][ $item['fakultaet_id'] ] = Array( 'name' => $fakultaet_name, 'evaluations' => Array() );
						}
						array_push( $result['running']['participated'][ $item['semester'] ][ $item['fakultaet_id'] ]['evaluations'], $item );
					}
					else
					{
						if ( empty( $result['running']['not_participated'][ $item['semester'] ] ) )
						{
							$result['running']['not_participated'][ $item['semester'] ] = Array();
						}
						if ( empty( $result['running']['not_participated'][ $item['semester'] ][ $item['fakultaet_id'] ] ) )
						{
							$fakultaet_name = ibit_Helper::getFacultyName( $item['fakultaet_id'] );
							$result['running']['not_participated'][ $item['semester'] ][ $item['fakultaet_id'] ] = Array( 'name' => $fakultaet_name, 'evaluations' => Array() );
						}
						array_push( $result['running']['not_participated'][ $item['semester'] ][ $item['fakultaet_id'] ]['evaluations'], $item );
					}
				}

			}
			else
			{
				if ( empty( $result['expired'][ $item['semester'] ] ) )
				{
					$result['expired'][ $item['semester'] ] = Array();
				}
				if ( empty( $result['expired'][ $item['semester'] ][ $item['fakultaet_id'] ] ) )
				{
					$fakultaet_name = ibit_Helper::getFacultyName( $item['fakultaet_id'] );
					$result['expired'][ $item['semester'] ][ $item['fakultaet_id'] ] = Array( 'name' => $fakultaet_name, 'evaluations' => Array() );
				}
				if ( $dozent or ( $item['participated'] ) )
				{
					array_push( $result['expired'][ $item['semester'] ][ $item['fakultaet_id'] ]['evaluations'], $item );
				}
			}
		}

		return $result;
	}

	/**
	 * Prft, ob ein User fr eine Evaluation zugelassen ist.
	 * @param string $user_id Id des Users
	 * @param int $evaluation_id Id der Evaluation
	 * @return boolean Gibt an, ob der User fr die Evaluation zugelassen ist.
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function userValidForEvaluation( $user_id, $evaluation_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql_eval = 'SELECT seminar_id FROM ibit_evaluation WHERE evaluation_id = ?';
		$sql_user = 'SELECT COUNT(*) AS valid FROM seminar_user WHERE Seminar_id = ? AND user_id = ?';

		$eval_result = $conn->Execute( $sql_eval, $evaluation_id );
		if ( !$eval_result or $eval_result->EOF)
		{
			return false;
		}

		$seminar_id = $eval_result->fields['seminar_id'];
		$eval_result->Close();

		$user_result = $conn->Execute( $sql_user, Array( $seminar_id, $user_id ) );
		if ( !$user_result or $user_result->EOF)
		{
			return false;
		}

		$return = $user_result->fields['valid'];
		$user_result->Close();

		return $return;
	}

	/**
	 * Prft, ob ein User an einer Evaluation teilgenommen hat.
	 * @param string $user_id Id des Users
	 * @param int $evaluation_id Id der Evaluation
	 * @return boolean Gibt an, ob ein User an einer Evaluation teilgenommen hat.
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function userParticipatedInEvaluation( $user_id, $evaluation_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = "SELECT COUNT(*) AS participated FROM ibit_evaluation_user WHERE user_id = ? AND evaluation_id = ?";

		$result = $conn->Execute( $sql, Array( $user_id, $evaluation_id ) );
		if ( !$result or $result->EOF)
		{
			return false;
		}

		$return = $result->fields['participated'];

		$result->Close();

		return $return;
	}

	/**
	 * Ldt die Evaluationen inklusive der Antworten eines Semesters.
	 * @param string $semester_id Id des Semesters
	 * @param int $min_users Minimale Anzahl an teilnehmenden Studenten
	 * @param boolean $visible Gibt an, ob nur freigeschaltete Evaluationen ausgegeben werden sollen
	 * @return mixed false bei Fehlern, ansonsten ein Array mit den Daten
	 * @static
	 * @uses ibit_DatabaseConnection
	 * @todo Dokumentation
	 */
	function getEvaluationAnswers( $semester_id, $min_users, $visible = true )
	{
		$conn = isset( $this->conn ) ? $this->conn : new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$evaluation_ids = Array();

		# Evaluationen auslesen
		$evaluations = Array();
		$sql_load_evaluations = 'SELECT e.evaluation_id, seminar_id, institut_id, fakultaet_id, SUM(u.hidden<>1) AS user_count FROM ibit_evaluation AS e LEFT JOIN ibit_evaluation_user AS u USING( evaluation_id ) WHERE ##additional_where_clause## stop_date < NOW() AND semester_id = ? GROUP BY e.evaluation_id';

		$additional_where_clause = '';
		if ( $visible )
		{
			$additional_where_clause = 'visible_date IS NOT NULL AND visible_date <= NOW() AND visible_author_id IS NOT NULL AND';
		}
		$sql_load_evaluations = str_replace( '##additional_where_clause##', $additional_where_clause, $sql_load_evaluations );

		if ( !$evaluation_result = $conn->Execute( $sql_load_evaluations, $semester_id ) )
		{
			return false;
		}
		while ( !$evaluation_result->EOF )
		{
			$evaluation = Array();
			$evaluation['id'] = $evaluation_result->fields['evaluation_id'];
			$evaluation['seminar_id'] = $evaluation_result->fields['seminar_id'];
			$evaluation['institut_id'] = $evaluation_result->fields['institut_id'];
			$evaluation['fakultaet_id'] = $evaluation_result->fields['fakultaet_id'];
			$evaluation['participants'] = $evaluation_result->fields['user_count'];
			$evaluation['groups'] = Array();
			$evaluations[ $evaluation['id'] ] = $evaluation;

			if ( $evaluation['participants'] >= $min_users )
			{
				array_push( $evaluation_ids, $evaluation['id'] );
			}

			$evaluation_result->MoveNext();
		}
		$evaluation_result->Close();

		if ( empty( $evaluation_ids ) )
		{
			return false;
		}

		$evaluation_string = implode(',', $evaluation_ids );

		$group_ids = Array();
		$question_ids = Array();
		$answer_ids = Array();

		# Struktur laden
		$sql_load_structure = 'SELECT evaluation_id, evaluation_group_template_id, evaluation_question_template_id, group_position, question_position FROM ibit_evaluation_links AS l INNER JOIN ibit_evaluation USING( evaluation_template_id ) WHERE evaluation_id IN ('.$evaluation_string.') ORDER BY group_position ASC, question_position ASC';
		if ( !$structure_result = $conn->Execute( $sql_load_structure ) )
		{
			return false;
		}
		while ( !$structure_result->EOF )
		{
			$evaluation_id = $structure_result->fields['evaluation_id'];
			$group_id = $structure_result->fields['evaluation_group_template_id'];
			$question_id = $structure_result->fields['evaluation_question_template_id'];
			$group_position = $structure_result->fields['group_position'];
			$question_position = $structure_result->fields['question_position'];
			if ( empty( $evaluations[ $evaluation_id ]['groups'][ $group_id ] ) )
			{
				$evaluations[ $evaluation_id ]['groups'][ $group_id ] = Array( 'group_id' => $group_id, 'position' => $group_position, 'questions' => Array() );
				array_push( $group_ids, $group_id );
			}
			if ( empty( $evaluations[ $evaluation_id ]['groups'][ $group_id ]['questions'][ $question_id ] ) )
			{
				$evaluations[ $evaluation_id ]['groups'][ $group_id ]['questions'][ $question_id ] = Array( 'question_id' => $question_id, 'position' => $question_position, 'answers' => Array() );
				array_push( $question_ids, $question_id );
			}
			$structure_result->MoveNext();
		}
		$structure_result->Close();

		# Antworten auslesen
		$sql_load_answers = 'SELECT evaluation_id, evaluation_group_template_id, evaluation_question_template_id, evaluation_answer_template_id, TRIM(value) AS value, COUNT(*) AS anzahl FROM ibit_evaluation_answers WHERE evaluation_id IN ('.$evaluation_string.') GROUP BY evaluation_id, evaluation_group_template_id, evaluation_question_template_id, evaluation_answer_template_id, LOWER(TRIM(value))';
		if ( !$answer_result = $conn->Execute( $sql_load_answers ) )
		{
			return false;
		}
		while ( !$answer_result->EOF )
		{
			$evaluation_id = $answer_result->fields['evaluation_id'];
			$group_id = $answer_result->fields['evaluation_group_template_id'];
			$question_id = $answer_result->fields['evaluation_question_template_id'];
			$answer_id = $answer_result->fields['evaluation_answer_template_id'];

			array_push( $answer_ids, $answer_id );

			$answer = Array();
			$answer['answer_id'] = $answer_id;
			$answer['user_value'] = $answer_result->fields['value'];
			$answer['count'] = $answer_result->fields['anzahl'];
			array_push( $evaluations[ $evaluation_id ]['groups'][ $group_id ]['questions'][ $question_id ]['answers'], $answer );

			$answer_result->MoveNext();
		}
		$answer_result->Close();

		# Gruppendaten auslesen
		$group_string = "'".implode( "','", array_unique( $group_ids ) )."'";
		$sql_load_group_data = 'SELECT evaluation_group_template_id, title, description FROM ibit_evaluation_group_templates WHERE evaluation_group_template_id IN ('.$group_string.')';
		if ( !$group_result = $conn->Execute( $sql_load_group_data ) )
		{
			return false;
		}

		$group_data = Array();
		while ( !$group_result->EOF)
		{
			$group = Array();
			$group['title'] = $group_result->fields['title'];
			$group['description'] = $group_result->fields['description'];
			$group_data[ $group_result->fields['evaluation_group_template_id'] ] = $group;

			$group_result->MoveNext();
		}
		$group_result->Close();

		# Fragendaten auslesen
		$question_string = "'".implode( "','", array_unique( $question_ids ) )."'";
		$sql_load_question_data = 'SELECT evaluation_question_template_id, q.title AS title, q.text AS text, t.title AS answer_title, t.description AS answer_description, t.html_type AS html_type FROM ibit_evaluation_question_templates AS q LEFT JOIN ibit_evaluation_answer_types AS t USING( evaluation_answer_type_id ) WHERE evaluation_question_template_id IN ('.$question_string.')';
		if ( !$question_result = $conn->Execute( $sql_load_question_data ) )
		{
			return false;
		}

		$question_data = Array();
		while (!$question_result->EOF)
		{
			$question = Array();
			$question['title'] = $question_result->fields['title'];
			$question['text'] = $question_result->fields['text'];
			$question['answer_title'] = $question_result->fields['answer_title'];
			$question['answer_description'] = $question_result->fields['answer_description'];
			$question['html_type'] = $question_result->fields['html_type'];
			$question_data[ $question_result->fields['evaluation_question_template_id'] ] = $question;

			$question_result->MoveNext();
		}
		$question_result->Close();

		# Antwortdaten auslesen
		$answer_string = "'".implode( "','", array_unique( $answer_ids ) )."'";
		$sql_load_answer_data = 'SELECT evaluation_answer_template_id, text, position, value FROM ibit_evaluation_answer_templates WHERE evaluation_answer_template_id IN ('.$answer_string.')';
		if ( !$answer_result = $conn->Execute( $sql_load_answer_data ) )
		{
			return false;
		}

		$answer_data = Array();
		while ( !$answer_result->EOF)
		{
			$answer = Array();
			$answer['text'] = $answer_result->fields['text'];
			$answer['position'] = $answer_result->fields['position'];
			$answer['value'] = $answer_result->fields['value'];
			$answer_data[ $answer_result->fields['evaluation_answer_template_id'] ] = $answer;

			$answer_result->MoveNext();
		}
		$answer_result->Close();

		# Daten ins Array fllen
		foreach ( $evaluations as $evaluation_idx=>$evaluation )
		{
			if ( $evaluation['participants'] < $min_users )
			{
				continue;
			}

			foreach ( $evaluation['groups'] as $group_idx => $group )
			{
				$group['title'] = $group_data[ $group_idx ]['title'];
				$group['description'] = $group_data[ $group_idx ]['description'];
				foreach ( $group['questions'] as $question_idx => $question )
				{
					$question['title'] = $question_data[ $question_idx ]['title'];
					$question['text'] = $question_data[ $question_idx ]['text'];
					$question['answer_title'] = $question_data[ $question_idx ]['answer_title'];
					$question['answer_description'] = $question_data[ $question_idx ]['answer_description'];
					$question['html_type'] = $question_data[ $question_idx ]['html_type'];
					foreach ( $question['answers'] as $answer_idx => $answer )
					{
						$answer['text'] = $answer_data[ $answer['answer_id'] ]['text'];
						$answer['position'] = $answer_data[ $answer['answer_id'] ]['position'];
						$answer['value'] = $answer_data[ $answer['answer_id'] ]['value'];
						$question['answers'][ $answer_idx ] = $answer;
					}
					$group['questions'][ $question_idx ] = $question;
				}
				$evaluation['groups'][ $group_idx ] = $group;
			}
			$evaluations[ $evaluation_idx ] = $evaluation;
		}

		# Daten zurckgeben
		return $evaluations;
	}

	/**
	 * Liefert die Anzahl der Teilnehmer an einer Menge von Evaluationen zurck.
	 * @param array $evaluation_ids Ids der Evaluationen
	 * @return array Anzahl der Teilnehmer gruppiert nach Id der jeweiligen Evaluation
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getEvaluationParticipants( $evaluation_ids )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		if ( !is_array( $evaluation_ids ) )
		{
			$evaluation_ids = Array( $evaluation_ids );
		}
		$sql = 'SELECT evaluation_id, SUM(hidden<>1) AS participants FROM ibit_evaluation_user WHERE evaluation_id IN ('.implode(',', $evaluation_ids).') GROUP BY evaluation_id';
		if ( !$result = $conn->Execute( $sql ) )
		{
			return false;
		}

		$return = Array();
		while ( !$result->EOF )
		{
			$return[ $result->fields['evaluation_id'] ] = $result->fields['participants'];
			$result->MoveNext();
		}
		$result->Close();

		return $return;
	}

	/**
	 * Liest diverse statistische Informationen ber die Evaluationen eines Semesters aus.
	 * Die Rckgabe sieht dabei folgendermassen aus:
	 *
	 * <pre>
	 *    [
	 *     evaluation_count  => Anzahl der Evaluationen,
	 *     visible_count     => Anzahl der freigeschalteten Evaluationen,
	 *     not_visible_count => Anzahl der nicht freigeschalteten Evaluationen,
	 *     participant_count => Gesamtzahl der Teilnehmer an den Evaluationen
	 *    ]
	 * </pre>
	 */
	function getEvaluationStatisticsForSemester( $semester_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$return = Array();

		# Anzahl der Evaluation, der freigeschalteten und der nicht freigeschalteten Evaluationen des Semesters ermitteln
		$sql = 'SELECT COUNT(*) AS evaluation_count, SUM( visible_date IS NOT NULL ) AS visible_count, fakultaet_id, institut_id FROM ibit_evaluation WHERE semester_id = ? GROUP BY fakultaet_id, institut_id WITH ROLLUP';
		if ( $result = $conn->Execute( $sql, $semester_id ) )
		{
			while ( !$result->EOF )
			{
				if ( !isset( $return[ $result->fields['fakultaet_id'] ] ) )
				{
					$return[ $result->fields['fakultaet_id'] ] = Array();
				}
				if ( !isset($return[ $result->fields['fakultaet_id'] ][ $result->fields['institut_id'] ]) )
				{
					$return[ $result->fields['fakultaet_id'] ][ $result->fields['institut_id'] ] = Array();
				}
				$return[ $result->fields['fakultaet_id'] ][ $result->fields['institut_id'] ]['evaluation_count'] = $result->fields['evaluation_count'];
				$return[ $result->fields['fakultaet_id'] ][ $result->fields['institut_id'] ]['visible_count'] = $result->fields['visible_count'];
				$return[ $result->fields['fakultaet_id'] ][ $result->fields['institut_id'] ]['not_visible_count'] = $result->fields['evaluation_count'] - $result->fields['visible_count'];

				$result->MoveNext();
			}
			$result->Close();
		}

		# Anzahl der Teilnehmer an den Evaluation des Semester ermitteln
		$sql_participants = 'SELECT SUM(u.hidden<>1) AS participant_count, COUNT( DISTINCT user_id ) AS participant_unique_count, fakultaet_id, institut_id FROM ibit_evaluation_user AS u INNER JOIN ibit_evaluation AS e USING(evaluation_id) WHERE e.semester_id = ? GROUP BY fakultaet_id, institut_id WITH ROLLUP';
		if ( $result = $conn->Execute( $sql_participants, $semester_id ) )
		{
			while ( !$result->EOF )
			{
				if ( !isset( $return[ $result->fields['fakultaet_id'] ] ) )
				{
					$return[ $result->fields['fakultaet_id'] ] = Array();
				}
				if ( !isset($return[ $result->fields['fakultaet_id'] ][ $result->fields['institut_id'] ]) )
				{
					$return[ $result->fields['fakultaet_id'] ][ $result->fields['institut_id'] ] = Array();
				}
				$return[ $result->fields['fakultaet_id'] ][ $result->fields['institut_id'] ]['participant_count'] = $result->fields['participant_count'];
				$return[ $result->fields['fakultaet_id'] ][ $result->fields['institut_id'] ]['participant_count_unique'] = $result->fields['participant_unique_count'];

				$result->MoveNext();
			}
			$result->Close();
		}

		# Anzahl der Teilnehmer an den Seminaren der Evaluationen ermitteln
		$sql = "SELECT COUNT(*) AS seminar_members, COUNT( DISTINCT user_id ) AS seminar_members_unique, fakultaet_id, institut_id FROM ibit_evaluation AS e LEFT JOIN seminar_user AS u ON (e.seminar_id=u.seminar_id) WHERE e.semester_id = ? AND u.status NOT IN('dozent', 'tutor', 'admin') GROUP BY fakultaet_id, institut_id WITH ROLLUP";
		if ( $result = $conn->Execute( $sql, $semester_id ) )
		{
			while ( !$result->EOF )
			{
				if ( !isset( $return[ $result->fields['fakultaet_id'] ] ) )
				{
					$return[ $result->fields['fakultaet_id'] ] = Array();
				}
				if ( !isset($return[ $result->fields['fakultaet_id'] ][ $result->fields['institut_id'] ]) )
				{
					$return[ $result->fields['fakultaet_id'] ][ $result->fields['institut_id'] ] = Array();
				}
				$return[ $result->fields['fakultaet_id'] ][ $result->fields['institut_id'] ]['seminar_members'] = $result->fields['seminar_members'];
				$return[ $result->fields['fakultaet_id'] ][ $result->fields['institut_id'] ]['seminar_members_unique'] = $result->fields['seminar_members_unique'];

				$result->MoveNext();
			}
			$result->Close();
		}

		foreach ($return as $faculty_id=>$faculty_data)
		{
			foreach ($faculty_data as $institute_id=>$institute_data)
			{
				if (empty($institute_data['participant_count']))
				{
					$return[$faculty_id][$institute_id]['participant_count'] = 0;
					$return[$faculty_id][$institute_id]['participant_count_unique'] = 0;
				}
				if (empty($institute_data['seminar_members']))
				{
					$return[$faculty_id][$institute_id]['seminar_members'] = 0;
					$return[$faculty_id][$institute_id]['seminar_members_unique'] = 0;
				}
			}
		}

		return $return;
	}

	/**
	 * Liefert Infos einer Evaluation zu einer Veranstaltung zurck
	 * @param string $seminar_id Id der Veranstaltung
	 * @return Array Infos zur Evaluation (Teilnehmer, ID, Freischaltung)
	 */
	function getEvaluationBySeminar( $seminar_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$return = Array();

		# Anzahl der Teilnehmer an den Evaluation des Semester ermitteln
		$sql_participants = 'SELECT SUM(u.hidden<>1) AS participants, visible_date, e.evaluation_id AS evaluation_id FROM ibit_evaluation_user AS u INNER JOIN ibit_evaluation AS e USING(evaluation_id) WHERE e.seminar_id = ? GROUP BY e.evaluation_id';
		if ( $result = $conn->Execute( $sql_participants, $seminar_id ) )
		{
			while ( !$result->EOF )
			{
				$return = array (
					'participants' => $result->fields['participants'],
					'visible' =>  $result->fields['visible_date'],
					'evaluation_id' => $result->fields['evaluation_id']
				);
				$result->MoveNext();
			}
			$result->Close();

		}

		if( empty($return) )
		{
			$return = array ('status' => "keine Evaluation vorhanden");
		}

		return $return;

	}
	
	/**
	 * Not documented yet
	 * @static
	 */
	function getFacultyForSeminar($seminar_id)
	{
		$conn = new ibit_DatabaseConnection(DISPLAY_SQL_ERRORS);
		
		$result = $conn->Execute('SELECT fakultaet_id FROM ibit_evaluation WHERE seminar_id=?', $seminar_id);
		
		if (!$result or $result->EOF)
		{
				return null;
		}
		$faculty_id = $result->fields['fakultaet_id'];
		
		$result->Close();
		$conn->Close();
		return $faculty_id;
	}
}
?>