<?php
/**
 * classes/ibit_EvaluationGroup.class.php
 *
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  Ivn Marcos <ivan.marcos@gmx.net>
 * @author  Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package IBIT_Evaluation
 * @copyright Copyright 2006, 2007 IBIT
 * @version 1.1
 */

/**
 * Die Reprsentation einer Evaluationsgruppe.
 *
 * Diese Klasse stellt eine Reprsentation der Daten einer Gruppe einer Evaluation dar.
 *
 * @author     Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author     Michael Riehemann <michael.riehemann@mail.uni-oldenburg.de>
 * @author     Ivn Marcos <ivan.marcos@gmx.net>
 * @package    IBIT_Evaluation
 * @subpackage Structure
 * @copyright  Copyright 2006, 2007 IBIT
 * @version    1.1
 */
class ibit_EvaluationGroup
{
	/**
	 * Datenbankverbindungsobjekt
	 * @var object $conn
	 */
	var $conn = null;

	/**
	 * SQL-Statement zum Laden einer Gruppe
	 * @var string $sql_load_evaluation_group
	 */
	var $sql_load_evaluation_group = 'SELECT title, description, change_date, protected FROM ibit_evaluation_group_templates WHERE evaluation_group_template_id = ?';

  /**
   * SQL-Statement zum ndern einer Gruppe
   * @var string $sql_update_evaluation_group
   */
  var $sql_update_evaluation_group = 'UPDATE ibit_evaluation_group_templates SET title=?, description=?, change_date=NOW(), protected=? WHERE evaluation_group_template_id = ?';

  /**
   * SQL-Statement zum Einfgen einer Gruppe
   * @var string $sql_insert_evaluation_group
   */
  var $sql_insert_evaluation_group = 'INSERT INTO ibit_evaluation_group_templates (title, description, change_date, protected) VALUES (?, ?, NOW(), ?)';

	/**
	 * Die ID der Gruppe
	 * @var string $evaluation_group_template_id
	 */
	var $evaluation_group_template_id = null;

	/**
	 * Array, das die Fragen der Gruppe enthlt
	 * @var array $questions
	 */
	var $questions = Array();

	/**
	 * Der Titel der Gruppe
	 * @var string $title
	 */
	var $title;

	/**
	 * Die Beschreibung der Gruppe
	 * @var string $description
	 */
	var $description;

	/**
	 * Der Geschtzt-Status der Evaluation
	 * @var int $protected
	 */
	var $protected;

	/**
	 * Das Datum der letzten nderung
	 * @var int $change_date
	 */
	var $change_date;

	/**
	 * Position der Gruppe innerhalb des Templates
	 * @var int
	 */
	var $position;

	/**
	 * Konstruktor der Klasse.
	 * @param int $id Id der Evaluationsgruppe
	 * @uses ibit_DatabaseConnection
	 */
	function ibit_EvaluationGroup( $id = null )
	{
		$this->conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		if ( $id !== null )
		{
			$this->load( $id );
		}
	}

	/**
	 * Ldt die Daten der Gruppe mit der bergebenen Id in dieses Objekt.
	 * @param string $eval_group_template_id Id der Evaluationsgruppe
	 * @return boolean Gibt an, ob das Laden erfolgreich war oder nicht.
	 */
	function load( $eval_group_template_id )
	{
		$this->evaluation_group_template_id = $eval_group_template_id;
		$loadResult = $this->conn->Execute( $this->sql_load_evaluation_group, $eval_group_template_id );
		if (!$loadResult or $loadResult->EOF)
		{
			return false;
		}
		$this->title = $loadResult->fields['title'];
		$this->description = $loadResult->fields['description'];
		$this->protected = $loadResult->fields['protected'];
		$this->change_date = $loadResult->fields['change_date'];
		$loadResult->Close();
		return true;
	}

	/**
	 * Speichert die Daten der Gruppe in der Datenbank.
	 */
	function save()
	{
		$data = Array();
		array_push( $data, $this->title );
		array_push( $data, $this->description );
		array_push( $data, $this->protected );
		if ( $this->evaluation_group_template_id === null)
		{
			$this->conn->Execute( $this->sql_insert_evaluation_group, $data );
			$this->evaluation_group_template_id = $this->conn->Insert_Id();
		}
		else
		{
			array_push( $data, $this->evaluation_group_template_id );
			$this->conn->Execute( $this->sql_update_evaluation_group, $data );
		}
	}

	/**
 	 * Fgt der Gruppe eine neue Frage hinzu.
	 * Es wird dabei verhindert, dass identische Fragen doppelt in einer Gruppe auftauchen.
	 * @param object $question_object Fragenobjekt, das der Evaluationsgruppe hinzugefgt werden soll
	 */
	function addQuestion( $question_object )
	{
		$found = false;
		if ( $question_object->getId() !== null )
		{
			foreach ( $this->questions as $question_index => $question )
			{
				if ( $question->getId() == $question_object->getId() )
				{
					$found = true;
					$this->questions[ $question_index ] = $question_object;
					break;
				}
			}
		}

		if ( !$found )
		{
			array_push( $this->questions, $question_object );
		}
	}

	/**
 	 * Lscht eine Frage aus der Gruppe.
	 * @param string $question_object Fragenobjekt, das aus der Gruppe gelscht werden soll.
	 */
	function removeQuestion( $question_object )
	{
		foreach ( $this->questions as $question_index => $question )
		{
			if ( $question_object->getId() == $question->getId() )
			{
				unset( $this->questions[ $question_index ] );
			}
		}
	}

	/**
	 * Liefert die Id der Evaluationsgruppe zurck.
	 * @return int Id der Evaluationsgruppe
	 */
	function getEvaluationGroupId()
	{
		return $this->evaluation_group_template_id;
	}

	/**
	 * Liefert die Id der Evaluationsgruppe zurck.
	 * @return int Id der Evaluationsgruppe
	 */
	function getId()
	{
		return $this->getEvaluationGroupId();
	}

	/**
	 * Setzt die Id der Evaluationsgruppe.
	 * @param int $evaluation_group_template_id Id der Evaluationsgruppe
	 */
	function setEvaluationGroupId( $evaluation_group_template_id )
	{
		$this->evaluation_group_template_id = $evaluation_group_template_id;
	}

	/**
	 * Setzt den Titel der Evaluationsgruppe.
	 * @param string $title Titel der Evaluationsgruppe
	 */
	function setTitle( $title )
	{
		$this->title = $title;
	}

	/**
	 * Liefert den Titel der Evaluationsgruppe zurck.
	 * @return string Titel der Evaluationsgruppe
	 */
	function getTitle()
	{
		return $this->title;
	}

	/**
	 * Setzt die Beschreibung der Evaluationsgruppe.
	 * @param string $description Beschreibung der Evaluationsgruppe
	 */
	function setDescription( $description )
	{
		$this->description = $description;
	}

	/**
	 * Liefert die Beschreibung der Evaluationsgruppe zurck.
	 * @return string Beschreibung der Evaluationsgruppe
	 */
	function getDescription()
	{
		return $this->description;
	}

	/**
	 * Liefert den Geschtzt-Status der Evaluationsgruppe zurck.
	 * @return int Geschtzt-Status der Evaluationsgruppe
	 */
	function getProtected()
	{
		return $this->protected;
	}

	/**
	 * Setzt den Geschtzt-Status der Evaluationsgruppe.
	 * @param int $protected Geschtzt-Status der Evaluationsgruppe
	 */
	function setProtected( $protected = 1 )
	{
		$this->protected = $protected;
	}

	/**
	 * Liefert das nderungsdatum der Evaluationsgruppe zurck.
	 * @return int nderungsdatum der Evaluationsgruppe als Unix-Timestamp
	 */
	function getChangeDate()
	{
		return $this->change_date;
	}

	/**
	 * Liefert die Position der Evaluationsgruppe innerhalb der Evaluation zurck.
	 * @return int Die Position der Evaluationsgruppe
	 */
	function getPosition()
	{
		return $this->position;
	}

	/**
	 * Setzt die Position der Evaluationsgruppe innerhalb der Evaluation.
	 * @param int $position Die Position der Evaluationsgruppe
	 */
	function setPosition( $position )
	{
		$this->position = $position;
	}

	/**
	 * Bricht das Objekt in ein mehrdimensionales Array inklusive der Fragen und Antworten der Evaluationsgruppe zurck.
	 * Die Rckgabe sieht dabei folgendermassen aus:<br/>
	 * <pre>
	 *    [
	 *     id          => Id der Evaluationsgruppe,
	 *     title       => Titel der Evaluationsgruppe,
	 *     description => Beschreibung der Evaluationsgruppe,
	 *     protected   => Geschtzt-Status der Evaluationsgruppe,
	 *     position    => Position der Evaluationsgruppe,
	 *     questions   => [
	 *       ...
	 *                    ]{0..x}
	 *    ]
	 * </pre>
	 *
	 * @return array Evaluationsgruppendaten
	 * @see ibit_EvaluationQuestion::flattenToArray();
	 */
	function flattenToArray()
	{
		$result = Array();
		$result['id'] = $this->evaluation_group_template_id;
		$result['title'] = $this->title;
		$result['description'] = $this->description;
		$result['protected'] = $this->protected;
		$result['position'] = $this->position;

		$result['questions'] = Array();
		foreach ( $this->questions as $question )
		{
			array_push( $result['questions'], $question->flattenToArray() );
		}

		return $result;
	}
}
?>