<?php
/**
 * classes/ibit_EvaluationTemplate.class.php
 *
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  Ivn Marcos <ivan.marcos@gmx.net>
 * @author  Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package IBIT_Evaluation
 * @copyright Copyright 2006, 2007 IBIT
 * @version 1.1
 */

/**
 * Die Reprsentation einer Evaluationsvorlage
 *
 * Diese Klasse stellt die Reprsentation der Daten einer Evaluationsvorlage dar. Es werden Funktionen zum Laden, Speichern
 * und Bearbeiten der Daten einer Vorlage angeboten.
 *
 * @author     Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author     Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author     Ivn Marcos <ivan.marcos@gmx.net>
 * @package    IBIT_Evaluation
 * @subpackage Structure
 * @copyright  Copyright 2006, 2007 IBIT
 * @version    1.1
 */
class ibit_EvaluationTemplate
{
	/**
	 * Datenbankverbindungsobjekt
	 * @var object $conn
	 */
	var $conn = null;

	/**
	 * Die ID der Evaluationsvorlage
	 * @var int $template_id
	 */
	var $template_id = null;

	/**
	 * Der Titel der Evaluationsvorlage
	 * @var string $title
	 */
	var $title;

	/**
	 * Die Beschreibung der Evaluationsvorlage
	 * @var string $description
	 */
	var $description;

	/**
	 * Das Datum der letzten nderung der Evaluationsvorlage
	 * @var int $change_date
	 */
	var $change_date;

	/**
	 * Der Status der Evaluationsvorlage
	 * @var int $status
	 */
	var $status;

	/**
	 * Gibt an, ob die Evaluationsvorlage vor nderungen geschtzt ist
	 * @var int $protected
	 */
	var $protected;

	/**
	 * Die Id des Autors der Evaluationsvorlage
	 * @var string $author_id
	 */
	var $author_id;

	/**
	 * Der Name des Autors der Evaluationsvorlage
	 * @var string $autor_name
	 */
	var $author_name;

	/**
 	 * Array, das die Gruppen der Evaluationsvorlage enthlt
	 * @var array $groups
	 */
	var $groups = Array();

	/**
	 * SQL-Statement zum Auslesen einer Evaluationsvorlage
	 * @var string $sql_load_template
	 */
	var $sql_load_template = "SELECT evaluation_template_id, title, description, change_date, status, protected, author_id, CONCAT(Vorname, ' ', Nachname) AS author_name FROM ibit_evaluation_templates AS t LEFT JOIN auth_user_md5 AS u ON (t.author_id COLLATE latin1_general_ci = u.user_id COLLATE latin1_general_ci) WHERE evaluation_template_id = ?";

	/**
	 * SQL-Statement zum Einfgen einer Evaluationsvorlage
	 * @var string $sql_insert_evaluation_template
	 */
	var $sql_insert_evaluation_template = 'INSERT INTO ibit_evaluation_templates (title, description, change_date, status, protected, author_id) VALUES (?, ?, NOW(), ?, ?, ?)';

	/**
	 * SQL-Statement zum ndern einer Evaluationsvorlage
	 * @var string $sql_update_evaluation_template
	 */
	var $sql_update_evaluation_template = 'UPDATE ibit_evaluation_templates SET title = ?, description = ?, change_date = NOW(), status = ?, protected = ?, author_id = ? WHERE evaluation_template_id = ?';

	/**
	 * SQL-Statement zum Auslesen der Struktur einer Evaluationsvorlage
	 * @var string $sql_load_structure
	 */
	var $sql_load_structure = 'SELECT evaluation_group_template_id, group_position, evaluation_question_template_id, question_position FROM ibit_evaluation_links WHERE evaluation_template_id = ? ORDER BY group_position ASC, question_position ASC';

	/**
	 * SQL-Statement zum Entfernen der Struktur einer Evaluationsvorlage
	 * @var string $sql_remove_structure
	 */
	var $sql_remove_structure = 'DELETE FROM ibit_evaluation_links WHERE evaluation_template_id = ?';

	/**
	 * SQL-Statement zum Speichern der Struktur einer Evaluationsvorlage
	 * @var string $sql_store_structure
	 */
	var $sql_store_structure = 'INSERT INTO ibit_evaluation_links (evaluation_template_id, evaluation_group_template_id, group_position, evaluation_question_template_id, question_position) VALUES (?, ?, ?, ?, ?)';

	/**
	 * Der Konstruktor der Klasse.
	 * @param int $id Id der zu ladenden Evaluationsvorlage
	 * @uses ibit_DatabaseConnection
	 */
	function ibit_EvaluationTemplate( $id = null )
	{
		// get connection
		$this->conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		if ( $id !== null ) {
			$this->load( $id );
		}
	}

	/**
	 * Ldt eine Evaluationsvorlage anhand der bergebenen Id.
	 * @param int $id Id der Evaluationsvorlage
	 * @return boolean Gibt an, ob das Laden erfolgreich war
	 */
	function load( $id )
	{
		$templateResult = $this->conn->Execute( $this->sql_load_template, $id );
		if ( !$templateResult or $templateResult->EOF )
		{
			return false;
		}

		$this->template_id = $id;
		$this->title = $templateResult->fields['title'];
		$this->description = $templateResult->fields['description'];
		$this->change_date = $templateResult->fields['change_date'];
		$this->status = $templateResult->fields['status'];
		$this->protected = $templateResult->fields['protected'];
		$this->author_id = $templateResult->fields['author_id'];
		$this->author_name = $templateResult->fields['author_name'];

		$templateResult->Close();

		$this->loadStructure();

		return true;
	}

	/**
	 * Speichert die Evaluationsvorlage in der Datenbank.
	 */
	function save()
	{
		$data = Array();
		array_push( $data, $this->title );
		array_push( $data, $this->description );
		array_push( $data, $this->status );
		array_push( $data, $this->protected );
		array_push( $data, $this->author_id );

		if ( $this->template_id === null )
		{
			$this->conn->Execute( $this->sql_insert_evaluation_template, $data );
			$this->template_id = $this->conn->Insert_Id();
		}
		else
		{
			array_push( $data, $this->template_id );
			$this->conn->Execute( $this->sql_update_evaluation_template, $data );
		}
	}

	/**
	 * Lscht eine Evaluationsvorlage und smtliche Querverweise aus der Datenbank.
	 * @param int $template_id Id der Evaluationsvorlage
	 * @return boolean Gibt an, ob das Lschen erfolgreich war
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function delete( $template_id )
	{
		$conn = isset( $this->conn ) ? $this->conn : new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql_delete_template = 'DELETE QUICK FROM ibit_evaluation_templates WHERE evaluation_template_id=? AND protected = 0';
		$sql_delete_structure = 'DELETE QUICK FROM ibit_evaluation_links WHERE evaluation_template_id=?';
		$sql_identify_abandoned_groups = 'SELECT t.evaluation_group_template_id AS group_id FROM ibit_evaluation_group_templates AS t LEFT JOIN ibit_evaluation_links AS l USING( evaluation_group_template_id ) WHERE l.evaluation_template_id IS NULL';
		$sql_identify_abandoned_questions = 'SELECT t.evaluation_question_template_id AS question_id FROM ibit_evaluation_question_templates AS t LEFT JOIN ibit_evaluation_links AS l USING( evaluation_question_template_id ) WHERE l.evaluation_template_id IS NULL';
		$sql_delete_abandoned_groups = 'DELETE QUICK FROM ibit_evaluation_group_templates WHERE evaluation_group_template_id IN (##IN_CLAUSE##)';
		$sql_delete_abandoned_questions = 'DELETE QUICK FROM ibit_evaluation_question_templates WHERE evaluation_question_template_id IN (##IN_CLAUSE##)';
		$sql_delete_optimize_after = 'OPTIMIZE TABLE ibit_evaluation_templates, ibit_evaluation_links';

		$conn->Execute( $sql_delete_template, $template_id ); //template lschen
		if ( $conn->Affected_Rows() == 0 )
		{
			return false;
		}
		$conn->Execute( $sql_delete_structure, $template_id ); //template-struktur lschen

		# herrenlose fragen und antworten lschen
		if ( $group_result = $conn->Execute( $sql_identify_abandoned_groups ) )
		{
			$group_ids = Array();
			while ( !$group_result->EOF )
			{

				array_push( $group_ids, $group_result->fields['group_id'] );
				$group_result->MoveNext();
			}
			$group_result->Close();

			if(!empty($group_ids))
			{
				$in_clause = implode(',', $group_ids );
				$sql = str_replace( '##IN_CLAUSE##', $in_clause, $sql_delete_abandoned_groups );
				$conn->Execute( $sql );
			}
		}

		if ( $question_result = $conn->Execute( $sql_identify_abandoned_questions ) )
		{
			$question_ids = Array();
			while ( !$question_result->EOF )
			{
				array_push( $question_ids, $question_result->fields['question_id'] );
				$question_result->MoveNext();
			}
			$question_result->Close();

			if(!empty($question_ids))
			{
				$in_clause = implode(',', $question_ids );
				$sql = str_replace( '##IN_CLAUSE##', $in_clause, $sql_delete_abandoned_questions );
				$conn->Execute( $sql );
			}
		}
		$conn->Execute( $sql_delete_optimize_after );

		return true;
	}

	/**
	 * Setzt den Titel der Evaluationsvorlage.
	 * @param string $title Titel der Evaluationsvorlage
	 */
	function setTitle( $title )
	{
		$this->title = $title;
	}

	/**
	 * Liefert den Titel der Evaluationsvorlage zurck.
	 * @return string Titel der Evaluationsvorlage
	 */
	function getTitle()
	{
		return $this->title;
	}

	/**
	 * Setzt die Beschreibung der Evaluationsvorlage.
	 * @param string $description Beschreibung der Evaluationsvorlage
	 */
	function setDescription( $description )
	{
		$this->description = $description;
	}

	/**
	 * Liefert die Beschreibung der Evaluationsvorlage zurck.
	 * @return string Beschreibung der Evaluationsvorlage
	 */
	function getDescription()
	{
		return $this->description;
	}

	/**
	 * Setzt die Id der Evaluationsvorlage.
	 * @param int $status Id der Evaluationsvorlage
	 */
	function setTemplateID( $id )
	{
		$this->template_id = $id;
	}

	/**
	 * Liefert die Id der Evaluationsvorlage zurck.
	 * @return int Id der Evaluationsvorlage
	 */
	function getTemplateID()
	{
		return $this->template_id;
	}

	/**
	 * Setzt den Status der Evaluationsvorlage.
	 * @param int $status Status der Evaluationsvorlage
	 */
	function setStatus( $status )
	{
		$this->status = $status;
	}

	/**
	 * Liefert den Status der Evaluationsvorlage zurck.
	 * @return int Status der Vorlage
	 */
	function getStatus()
	{
		return $this->status;
	}

	/**
	 * Setzt das Protected-Flag der Evaluationsvorlage.
	 * @param boolean $protected Protected-Flag der Evaluationsvorlage
	 */
	function setProtected( $protected = 1 )
	{
		$this->protected = $protected;
	}

	/**
	 * Liefert das Protected-Flag der Evaluationsvorlage zurck.
	 * @return boolean Protected-Flag der Evaluationsvorlage
	 */
	function getProtected()
	{
		return $this->protected;
	}

	/**
	 * Setzt die Id des Autoren der Evaluationsvorlage
	 * @param string $author_id Id des Autoren
	 */
	function setAuthorId( $author_id )
	{
		$this->author_id = $author_id;
	}

	/**
	 * Liefert die Id des Autoren der Evaluationsvorlage zurck.
	 * @return string Id des Autoren
	 */
	function getAuthorId()
	{
		return $this->author_id;
	}

	/**
	 * Liefert den Namen des Autoren der Evaluationsvorlage zurck.
	 * @return string Name des Autoren
	 */
	function getAuthorName()
	{
		return $this->author_name;
	}

	/**
	 * Fgt eine Gruppe der Evaluationsvorlage hinzu.
	 * Es wird verhindert, dass zwei identische Gruppen vorhanden sind.
	 * @param object $group_object Gruppenobjekt, das der Evaluationsvorlage hinzugefgt werden soll
	 */
	function addGroup( $group_object )
	{
		$found = false;
		if ( $group_object->getId() !== null )
		{
			foreach ( $this->groups as $group_index => $group )
			{
				if ( $group->getId() == $group_object->getId() )
				{
					$found = true;
					$this->groups[ $group_index ] = $group_object;
					break;
				}
			}
		}
		if ( !$found )
		{
			array_push( $this->groups, $group_object );
		}
	}

	/**
	 * Entfernt eine Gruppe aus der Evaluationsvorlage.
	 * @param object $group_object Gruppenobjekt, das aus der Evaluationsvorlage entfernt werden soll
	 */
	function removeGroup( $group_object )
	{
		foreach ( $this->groups as $group_index => $group )
		{
			if ( $group_object->getEvaluationGroupId() == $group->getEvaluationGroupId() )
			{
				unset( $this->groups[ $group_index ] );
			}
		}
	}

	/**
	 * Ldt die komplette Struktur der Evaluationsvorlage aus der Datenbank.
	 * @return boolean Gibt an, ob das Laden erfolgreich war
	 * @uses ibit_EvaluationGroup
	 * @uses ibit_EvaluationQuestion
	 * @todo Bei dieser Funktion wre zu berlegen, sie neu zu schreiben und smtliche Queries hier auszufhren, um den SQL-Traffic auf
	 *       4 Queries im besten Fall zu reduzieren.
	 */
	function loadStructure()
	{
		$groups_loaded = Array();
		$this->groups = Array();

		$structure_result = $this->conn->Execute( $this->sql_load_structure, $this->template_id );

		if ( !$structure_result )
		{
			return false;
		}

		$group = null;
		while ( !$structure_result->EOF )
		{
			if ( !in_array( $structure_result->fields['evaluation_group_template_id'], $groups_loaded ) )
			{
				array_push( $groups_loaded, $structure_result->fields['evaluation_group_template_id'] );

				if ( $group !== null )
				{
					array_push( $this->groups, $group );
				}
				$group = new ibit_EvaluationGroup( $structure_result->fields['evaluation_group_template_id'] );
				$group->setPosition( $structure_result->fields['group_position'] );
			}

			$question = new ibit_EvaluationQuestion( $structure_result->fields['evaluation_question_template_id'] );
			$question->setPosition( $structure_result->fields['question_position'] );
			$question->loadAnswers();
			$group->addQuestion( $question );

			$structure_result->MoveNext();
		}
		$structure_result->Close();

		array_push( $this->groups, $group );

		return true;
	}

	/**
	 * Speichert die Struktur der Evaluationsvorlage in der Datenbank.
	 */
	function storeStructure()
	{
		if ( $this->template_id === null )
		{
			$this->save();
		}
		$this->conn->Execute( $this->sql_remove_structure, $this->template_id );

		foreach ( $this->groups as $group )
		{
			$group->save();

			foreach ( $group->questions as $question )
			{
				$question->save();

				$data = Array();
				array_push( $data, $this->template_id );
				array_push( $data, $group->getId() );
				array_push( $data, $group->getPosition() );
				array_push( $data, $question->getId() );
				array_push( $data, $question->getPosition() );

				$this->conn->Execute( $this->sql_store_structure, $data );
			}
		}
	}

	/**
	 * Bricht das Objekt der Evaluationsvorlage in ein Array auf.
	 * Die Rckgabe sieht dabei folgendermassen aus:<br/>
	 * <pre>
	 *    [
	 *     id          => Id der Evaluationsvorlage,
	 *     title       => Titel der Evaluationsvorlage,
	 *     description => Beschreibung der Evaluationsvorlage,
	 *     status      => Status der Evaluationsvorlage,
	 *     change_date => nderungsdatum der Evaluationsvorlage,
	 *     protected   => Gibt an, ob die Evaluationsvorlage vor Vernderungen geschtzt ist,
	 *     author_id   => Id des Autoren der Evaluationsvorlage,
	 *     author_name => Name des Autoren der Evaluationsvorlage,
	 *     groups      => [
	 *       ...
	 *                    ]{0..x}
	 *    ]
	 * </pre>
	 *
	 * @return array Evaluationsvorlagendaten
	 * @see ibit_EvaluationGroup::flattenToArray()
	 */
	function flattenToArray()
	{
		$result = Array();
		$result['id'] = $this->template_id;
		$result['title'] = $this->title;
		$result['description'] = $this->description;
		$result['status'] = $this->status;
		$result['change_date'] = $this->change_date;
		$result['protected'] = $this->protected;
		$result['author_id'] = $this->author_id;
		$result['author_name'] = $this->author_name;

		$result['groups'] = Array();

		foreach ( $this->groups as $group )
		{
			array_push( $result['groups'], $group->flattenToArray() );
		}
		return $result;
	}

##########################
## STATISCHE FUNKTIONEN ##
##########################

	/**
	 * Ldt Infos ber alle vorhandenen Evaluationen aus der Datenbank.
	 * Die Rckgabe sieht dabei folgendermassen aus:
	 *    [
	 *     id          => Id der Evaluationsvorlage,
	 *     title       => Titel der Evaluationsvorlage,
	 *     description => Beschreibung der Evaluationsvorlage,
	 *     status      => Status der Evaluationsvorlage,
	 *     change_date => nderungsdatum der Evaluationsvorlage,
	 *     protected   => Gibt an, ob die Evaluationsvorlage vor Vernderungen geschtzt ist,
	 *     author_id   => Id des Autoren der Evaluationsvorlage,
	 *     author_name => Name des Autoren der Evaluationsvorlage
	 *    ]{0..x}
	 *
	 * @param boolean $active Gibt an, ob nur aktive Evaluationsvorlagen geladen werden sollen
	 * @return array Evaluationsvorlagendaten
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getTemplatesInfo( $active = false )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = "SELECT evaluation_template_id, title, description, change_date, author_id, status, protected, CONCAT(Vorname, ' ', Nachname) AS author_name FROM ibit_evaluation_templates AS t LEFT JOIN auth_user_md5 AS u ON (t.author_id COLLATE latin1_general_ci = u.user_id COLLATE latin1_general_ci)";
		if ( $active )
		{
			$sql .= ' WHERE t.status = 1';
		}

		if ( !$templates_result = $conn->Execute( $sql ) )
		{
			return Array();
		}

		$result = Array();
		while ( !$templates_result->EOF )
		{
			$info = Array();
			$info['id'] = $templates_result->fields['evaluation_template_id'];
			$info['title'] = $templates_result->fields['title'];
			$info['description'] = $templates_result->fields['description'];
			$info['change_date'] = $templates_result->fields['change_date'];
			$info['author_id'] = $templates_result->fields['author_id'];
			$info['protected'] = $templates_result->fields['protected'];
			$info['status'] = $templates_result->fields['status'];
			$info['author_name'] = $templates_result->fields['author_name'];

			array_push( $result, $info );

			$templates_result->MoveNext();
		}
		$templates_result->Close();

		$conn->Close();

		return $result;
	}
}
?>