<?php
/**
 * classes/ibit_Helper.class.php
 *
 * @author  Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author  Ivn Marcos <ivan.marcos@gmx.net>
 * @author  Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package IBIT_Evaluation
 * @copyright Copyright 2006, 2007 IBIT
 * @version 1.1
 */

/**
 * Generelle "Helper"-Klasse
 *
 * Diese Klasse fasst verschiedene statische Funktionen zusammen, die nicht wirklich in den Kontext einer anderen Klasse passen oder an
 * verschiedenen Stellen unterschiedlicher Klassen verwendet werden.
 *
 * @author     Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author     Ivn Marcos <ivan.marcos@gmx.net>
 * @author     Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @package    IBIT_Evaluation
 * @subpackage Helper
 * @copyright  Copyright 2006, 2007 IBIT
 * @version    1.7
 */
class ibit_Helper
{
	/**
	 * Ldt die Fakultts-Id eines Instituts aus der Datenbank.
	 * @param string $institute_id Id des Instituts
	 * @return array Fakultts-Id
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getFacultyForInstitute( $institute_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = 'SELECT fakultaets_id FROM Institute WHERE Institut_id = ? LIMIT 0,1';
		$result = $conn->Execute( $sql, $institute_id );

		if ( !$result or $result->EOF )
		{
			return false;
		}

		$faculty_id = $result->fields['fakultaets_id'];

		$result->Close();
		$conn->Close();

		return $faculty_id;
	}

	/**
	 * Ldt den Namen einer Fakultt aus der Datenbank.
	 * @param string $faculty_id Id der Fakultt
	 * @return string Name der Fakultt
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getFacultyName( $faculty_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = 'SELECT Name FROM Institute WHERE Institut_id = fakultaets_id AND fakultaets_id = ?';

		$result = $conn->Execute( $sql, $faculty_id );
		if ( !$result or $result->EOF )
		{
			return false;
		}

		$faculty_name = $result->fields['Name'];

		$result->Close();
		$conn->Close();

		return $faculty_name;
	}

	/**
	 * Ldt den Namen eines Instituts aus der Datenbank.
	 * @param string $institute_id Id des Instituts
	 * @return string Name des Instituts
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getInstituteName( $institute_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = 'SELECT Name FROM Institute WHERE Institut_id = ?';

		$result = $conn->Execute( $sql, $institute_id );
		if ( !$result or $result->EOF )
		{
			return false;
		}

		$name = $result->fields['Name'];

		$result->Close();
		$conn->Close();

		return $name;
	}

	/**
	 * Liefert die Anzahl an Instituten fr eine bestimmte Fakultt zurck.
	 *
	 * @param int $faculty_id Id der Fakultt
	 * @return int Anzahl der Institute
	 */
	function getInstituteCountByFacultyId( $faculty_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = 'SELECT COUNT(*) AS institute_count FROM Institute WHERE fakultaets_id = ?';
		if ( !$result = $conn->Execute( $sql, $faculty_id ) or $result->EOF )
		{
			return 0;
		}
		$count = $result->fields['institute_count'];
		$result->Close();

		return $count;
	}

	/**
	 * Liefert die Anzahl an Seminaren fr eine bestimmte Fakultt und ein bestimmtes Semester zurck.
	 *
	 * @param int $faculty_id Id der Fakultt
	 * @param string $semester_id Id des Semesters
	 * @return int Anzahl der Seminare
	 */
	function getSeminarCountByFacultyId( $faculty_id, $semester_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = 'SELECT COUNT( DISTINCT s.Seminar_id ) AS seminar_count FROM seminare AS s LEFT JOIN seminar_inst AS si ON (si.seminar_id=s.seminar_id) LEFT JOIN Institute AS i ON( i.Institut_id IN (s.Institut_id,si.institut_id) ) INNER JOIN semester_data AS sem ON (sem.semester_id=?) WHERE fakultaets_id=? AND start_time BETWEEN sem.beginn AND sem.ende';
		if ( !$result = $conn->Execute( $sql, Array($semester_id,$faculty_id) ) or $result->EOF )
		{
			return 0;
		}
		$count = $result->fields['seminar_count'];
		$result->Close();

		return $count;
	}

	/**
	 * Liefert die Anzahl an Seminaren fr ein bestimmtes Institut und ein bestimmtes Semester zurck.
	 *
	 * @param int $institute_id Id des Instituts
	 * @param string $semester_id Id des Semesters
	 * @return int Anzahl der Seminare
	 */
	function getSeminarCountByInstituteId( $institute_id, $semester_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = 'SELECT COUNT( DISTINCT s.Seminar_id ) AS seminar_count FROM seminare AS s LEFT JOIN seminar_inst AS si ON (si.seminar_id=s.seminar_id) LEFT JOIN Institute AS i ON( i.Institut_id IN (s.Institut_id,si.institut_id) ) INNER JOIN semester_data AS sem ON (sem.semester_id=?) WHERE i.Institut_id=? AND start_time BETWEEN sem.beginn AND sem.ende';
		if ( !$result = $conn->Execute( $sql, Array($semester_id,$institute_id) ) or $result->EOF )
		{
			return 0;
		}
		$count = $result->fields['seminar_count'];
		$result->Close();

		return $count;
	}

	/**
	 * Ldt den Namen eines Seminars aus der Datenbank.
	 * @param string $seminar_id Die Id des Seminars
	 * @return string Name des Seminars
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getSeminarName( $seminar_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		if ( !is_array( $seminar_id ) )
		{
			$sql = 'SELECT name FROM seminare WHERE Seminar_id = ?';
			$resultSet = $conn->Execute( $sql, $seminar_id );
			if ( !$resultSet or $resultSet->EOF )
			{
				return false;
			}
			$result = $resultSet->fields['name'];
		}
		else
		{
			$sql = "SELECT Seminar_id, name FROM seminare WHERE Seminar_id IN ('".implode( "','", $seminar_id )."')";
			$resultSet = $conn->Execute( $sql );
			if ( !$resultSet )
			{
				return false;
			}
			$result = Array();
			while ( !$resultSet->EOF )
			{
				$result[ $resultSet->fields['Seminar_id'] ] = $resultSet->fields['name'];
				$resultSet->MoveNext();
			}
		}
		$resultSet->Close();
		$conn->Close();

		return $result;
	}

	/**
	 * Liefert eine Liste aller vorhandenen Semester zurck.
	 * Die Rckgabe sieht dabei folgendermassen aus:<br/>
	 * <pre>
	 *    [
	 *     id      => Id des Semesters,
	 *     name    => Name des Semesters,
	 *     current => Gibt an, ob das Semester das aktuelle Semester ist
	 *    ]{0..x}
	 * </pre>
	 *
	 * @return array Semesterdaten
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getSemester()
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$result = $conn->Execute('SELECT semester_id, name, UNIX_TIMESTAMP() BETWEEN beginn AND ende AS current FROM semester_data ORDER BY beginn DESC');

		if ( !$result )
		{
			return false;
		}

		$semester = array();
		while ( !$result->EOF )
		{
			$this_semester = Array();
			$this_semester['id'] = $result->fields['semester_id'];
			$this_semester['name'] = $result->fields['name'];
			$this_semester['currentsemester'] = $result->fields['current'];

			array_push( $semester, $this_semester );

			$result->MoveNext();
		}
		$result->Close();
		$conn->Close();

		return $semester;
	}

	/**
	 * Ldt den Namen eines Semesters aus der Datenbank.
	 * @param string $semester_id Id des Semesters
	 * @return string Name des Semesters
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getSemesterName( $semester_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$result = $conn->Execute('SELECT name FROM semester_data WHERE semester_id = ?', $semester_id);
		if ( !$result or $result->EOF )
		{
			return false;
		}
		$name = $result->fields['name'];
		$result->Close();
		$conn->Close();

		return $name;
	}

	/**
	 * Liefert eine Liste aller Semester zurck, in denen (abgelaufene) Evaluationen vorhanden sind.
	 * Die Rckgabe sieht dabei folgendermassen aus:<br/>
	 * <pre>
	 *    [
	 *     id      => Id des Semesters,
	 *     name    => Name des Semesters,
	 *     current => Gibt an, ob das Semester das aktuelle Semester ist
	 *    ]{0..x}
	 * </pre>
	 *
	 * @param boolean $expired Gibt an, ob nur Semester mit abgelaufenen Evaluationen ausgelesen werden sollen
	 * @return array Semesterdaten
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getEvaluationSemester( $expired = false )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = 'SELECT DISTINCT semester_id FROM ibit_evaluation';
		if ( $expired )
		{
			$sql .= ' WHERE stop_date < NOW()';
		}

		if ( !$result = $conn->Execute( $sql ) )
		{
			return false;
		}

		$semester_ids = Array();
		while ( !$result->EOF )
		{
			array_push( $semester_ids, $result->fields['semester_id'] );
			$result->MoveNext();
		}
		$result->Close();

		if ( empty( $semester_ids ) )
		{
			return array();
		}

		$id_string = "'".implode( "','", $semester_ids )."'";
		$sql_semester = 'SELECT semester_id, name, UNIX_TIMESTAMP() BETWEEN beginn AND ende AS current FROM semester_data WHERE semester_id IN ('.$id_string.') ORDER BY beginn DESC';
		if ( !$semester_result = $conn->Execute( $sql_semester ) )
		{
			return false;
		}

		$semester = Array();
		while ( !$semester_result->EOF )
		{
			$this_semester = Array();
			$this_semester['id'] = $semester_result->fields['semester_id'];
			$this_semester['name'] = $semester_result->fields['name'];
			$this_semester['currentsemester'] = $semester_result->fields['current'];

			array_push( $semester, $this_semester );

			$semester_result->MoveNext();
		}
		$semester_result->Close();
		$conn->Close();

		return $semester;
	}

	/**
	 * Liefert eine Liste aller vorhandenen Fakultten zurck.
	 * Die Rckgabe sieht dabei folgendermassen aus:<br/>
	 * <pre>
	 *    [
	 *     fakultaet_id => Id der Fakultt,
	 *     name         => Name der Fakultt
	 *    ]{0..x}
	 * </pre>
	 *
	 * @return array Fakulttsdaten
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getFaculties()
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$result = $conn->Execute( "SELECT Institut_id, Name FROM Institute WHERE Institut_id = fakultaets_id ORDER BY Name ASC" );

		if ( !$result )
		{
			return false;
		}

		$faculties = Array();
		while ( !$result->EOF )
		{
			$faculty = Array();
			$faculty['fakultaet_id'] = $result->fields['Institut_id'];
			$faculty['name'] = $result->fields['Name'];

			array_push( $faculties, $faculty );

			$result->MoveNext();
		}
		$result->Close();
		$conn->Close();

		return $faculties;
	}

	/**
	 * Liefert eine Liste aller vorhandenen Institute zurck.
	 * Die Rckgabe sieht dabei folgendermassen aus:<br/>
	 * <pre>
	 *    [
	 *     institute_id => Id des Instituts,
	 *     name         => Name des Instituts
	 *    ]{0..x}
	 * </pre>
	 *
	 * @return array Institutdaten
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getInstitutes()
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$result = $conn->Execute( "SELECT Institut_id, Name FROM Institute WHERE Institut_id != fakultaets_id ORDER BY Name ASC" );

		if ( !$result )
		{
			return false;
		}

		$institutes = Array();
		while ( !$result->EOF )
		{
			$institute = Array();
			$institute['institute_id'] = $result->fields['Institut_id'];
			$institute['name'] = $result->fields['Name'];

			array_push( $institutes, $institute );

			$result->MoveNext();
		}
		$result->Close();
		$conn->Close();

		return $institutes;
	}

	/**
	 * Schtzt eine Evaluationsvorlage vor Vernderungen.
	 * @param int $template_id Die Id der zu schtzenden Evaluationsvorlage
	 * @static
	 * @uses ibit_EvaluationTemplate
	 */
	function protectTemplate( $template_id )
	{
		$template = new ibit_EvaluationTemplate( $template_id );

		foreach ($template->groups as $group)
		{
			$group->setProtected();
			$group->save();

			foreach ($group->questions as $question)
			{
				$question->setProtected();
				$question->save();
			}
		}
		$template->setProtected();
		$template->save();
	}

	/**
	 * Prft einen bergebenen numerischen Wert auf Gltigkeit.
	 *
	 * @param int $number Der zu prfende numerische Wert
	 * @param int $min Das Minimum des Wertes (null fr keine Angabe)
	 * @param int $max Das Maximum des Wertes (null fr keine Angabe)
	 * @param string $description Die Beschreibung des Wertes
	 * @param array $error Container fr Fehlermeldungen (Output-Parameter)
	 * @static
	 */
	function checkNumber( &$number, $min, $max, $description, &$error )
	{
		if ( !isset( $number ) )
		{
			array_push( $error, 'Bitte '.$description.' angeben' );
		}
		elseif ( !is_numeric( $number ) or ( $min !== null and $number < $min ) or ( $max !== null and $number > $max ) )
		{
			unset( $number );
			$error_string = 'Bitte '.$description.' korrekt angeben';
			if ( $min === null )
			{
				$error_string .= ' (<'.$max.')';
			}
			elseif ( $max === null )
			{
				$error_string .= ' (>'.$min.')';
			}
			elseif ( $min !== null and $max !== null )
			{
				$error_string .= ' ('.$min.'..'.$max.')';
			}
			array_push( $error,  $error_string );
		}
	}

	/**
	 * Liefert alle Institute aus der Datenbank als Objekte zurck.
	 * @return array Liste aller Institute als Objekte
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getInstituteObjects()
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );
		// Cache the query for 300 seconds
		$sql = "SELECT LPAD(Institut_id, 32, '0') AS inst_id, Institut_id, Name, fakultaets_id FROM Institute ORDER BY fakultaets_id, inst_id, Name";
		if ( !$result = $conn->Execute( $sql ) )
		{
			return false;
		}

		$institutes = array();
    	while ( !$result->EOF ){
    		$instid = $result->fields['Institut_id'];
    		$parentid = $result->fields['fakultaets_id'];
    		$name = $result->fields['Name'];

    		if ( $instid == $parentid )
			{
    			$institute = new StudIPInstitute();
    			$institute->setId($instid);
    			$institute->setName($name);
    			if ( isset( $institutes[$instid] ) and is_object( $institutes[$instid] ) )
				{
    				$children = $institutes[ $instid ]->getAllChildInstitutes();
    				foreach ( $children as $child )
					{
    					$institute->addChild( $child );
    				}
    			}
    			$institutes[ $instid ] = $institute;
    		}
    		else
			{
    			$child = new StudIPInstitute();
    			$child->setId($instid);
    			$child->setName($name);
    			if ( !isset( $institutes[$parentid] ) or !is_object( $institutes[$parentid] ) )
				{
    				$institute = new StudIPInstitute();
    			}
				else
				{
					$institute = $institutes[ $parentid ];
				}
    			$institute->addChild( $child );
    			$institutes[ $parentid ] = $institute;
    		}
    		$result->MoveNext();
    	}
    	$result->Close();

    	return $institutes;
	}

	/**
	 * Liefert ein Institut aus der Datenbank als Objekt zurck.
	 * @param string $institute_id Id des Instituts
	 * @return object Das Institut als Objekt
	 * @static
	 * @uses ibit_DatabaseConnection
	 * @uses ibit_Helper::getInstituteByFacultyId()
	 * @todo Bei Update auf PHP 5 Referenz auf ibit_Helper::getInstituteByFacultyId() durch self::getInstituteByFacultyId() ersetzen.
	 */
	function getInstituteObject( $institute_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = 'SELECT Institut_id, Name, fakultaets_id FROM Institute WHERE Institut_id=?';
		if ( ( !$result = $conn->Execute( $sql, $institute_id ) ) or $result->EOF )
		{
			return false;
		}

		$instid = $result->fields['Institut_id'];
		$parentid = $result->fields['fakultaets_id'];
		$name = $result->fields['Name'];

		$institute = new StudIPInstitute();
		$institute->setId( $instid );
		$institute->setName( $name );
		if ( $parentid == $instid )
		{
			$institutes = ibit_Helper::getInstituteByFacultyId( $parentid );
			foreach ( $institutes as $child )
			{
				$institute->addChild( $child );
			}
		}
    	$result->Close();

    	return $institute;
	}

	/**
	 * Liefert die zu einer Fakultten gehrenden Institute als Objekte zurck.
	 * @param string $faculty_id Id der Fakultt
	 * @return array Liste der Institute als Objekte
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getInstituteByFacultyId( $faculty_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = 'SELECT Institut_id, Name, fakultaets_id FROM Institute WHERE fakultaets_id=? AND Institut_id <> fakultaets_id';
		if ( !$result = $conn->Execute( $sql, $faculty_id ) )
		{
			return false;
		}

    	$institutes = Array();
    	while ( !$result->EOF ){
    		$instid = $result->fields['Institut_id'];
    		$name = $result->fields['Name'];

    		$institute = new StudIPInstitute();
    		$institute->setId( $instid );
    		$institute->setName( $name );

    		array_push( $institutes, $institute );
    		$result->moveNext();
    	}
    	$result->Close();

    	return $institutes;
	}

	/**
	 * Liefert die Seminare eines Instituts in einem bestimmten Semester zurck.
	 * Die Rckgabe sieht dabei wie folgt aus:
	 *
	 * <pre>
	 *    [
	 *      id    => Id des Seminars,
	 *      titel => Titel des Seminars
	 * 		typ   => Art der Veranstaltung
	 *    ]{0..x}
	 * </pre>
	 *
	 * @param string $institute_id Id des Instituts
	 * @param string $semester_id Id des Semesters
	 * @return array Liste der Seminare
	 * @static
	 * @uses ibit_DatabaseConnection
	 */
	function getSeminarsForInstitute( $institute_id, $semester_id )
	{
		$conn = new ibit_DatabaseConnection( DISPLAY_SQL_ERRORS );

		$sql = 'SELECT se.Seminar_id, se.Name, se.status FROM seminar_inst s JOIN seminare se ON (s.seminar_id=se.Seminar_id) WHERE s.institut_id=? AND se.start_time BETWEEN (SELECT beginn FROM semester_data WHERE semester_id=?) AND (SELECT ende FROM semester_data WHERE semester_id=?) UNION SELECT sem.Seminar_id, sem.Name, sem.status FROM seminare sem WHERE sem.institut_id=? AND sem.start_time BETWEEN (SELECT beginn FROM semester_data WHERE semester_id=?) AND (SELECT ende FROM semester_data WHERE semester_id=?)';
		if ( !$result = $conn->Execute( $sql, Array( $institute_id, $semester_id, $semester_id, $institute_id, $semester_id, $semester_id ) ) )
		{
			return false;
		}

    	$courses = array();
    	while ( !$result->EOF )
		{
			$course = Array();
			$course['id'] = $result->fields['Seminar_id'];
			$course['titel'] = $result->fields['Name'];
			$course['typ'] = $result->fields['status'];
    		array_push( $courses, $course );

    		$result->moveNext();
    	}
    	$result->Close();
    	return $courses;
	}
}
?>