<?php
/**
 * XMLWriter - Erzeugt simple eindimensionale XML-Dokumente aus Arrays
 *
 * Diese Klasse dient dazu, aus einem zwei-dimensionalen assoziativen Array
 * ein simples XML-Dokument zu erzeugen.
 *
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @package File_Functions
 * @version 1.0
 */

class XMLWriter
{
	/**
	 * Der Name der Ausgabedatei (ohne Erweiterung)
	 * @var string $file_name
	 */
	var $file_name = null;

	/**
	 * Der Name des Wurzelknotens
	 * @var string $root_name
	 */
	var $root_name = null;

	/**
	 * Der Name der einzelnen Zeilenknoten
	 * @var string $row_name
	 */
	var $row_name = null;

	/**
	 * Das verwendete Filehandle
	 * @var mixed $file_handle
	 */
	var $file_handle = null;

	/**
	 * Gibt an, ob Entitten enkodiert werden sollen
	 * @var boolean $encode_entities
	 */
	var $encode_entities;

	/**
	 * Gibt an, ob Duplikate verhindert werden sollen
	 * @var boolean $avoid_duplicates
	 */
	var $avoid_duplicates;

	/**
	 * Enthlt die schon gespeicherten Zeilen, um Duplikate zu vermeiden
	 * @var array $written_rows
	 */
	var $written_rows;

	/**
	 * Die maximale Gre der Datei bevor gesplittet wird (0 fr keine Einschrnkung)
	 * @var int $max_size
	 */
	var $max_size = 0;

	/**
	 * Bisherige Anzahl an geschriebenen Teilen
	 * @var int $written_parts
	 */
	var $written_parts = 0;

	/**
	 * Bisherige Gre der aktuellen Datei
	 * @var int $part_size
	 */
	var $part_size = 0;

	
	/**
	 * Konstruktor der Klasse
	 *
	 * @param String $file_name Name der Ausgabedatei (.xml wird angehngt, falls nicht vorhanden)
	 * @param String $root_name Name des Wurzelknotens (default: root)
	 * @param String $row_name Name der Zeilenknoten (default: row)
	 * @param boolean $avoid_duplicates Gibt an, ob Duplikate vermieden werden sollen
	 * @param boolean $encode_entities Gibt an, ob Entitten enkodiert werden sollen
	 */
	function XMLWriter( $file_name, $root_name = 'root', $row_name = 'row', $avoid_duplicates = false, $encode_entities = false )
	{
		$this->root_name = $root_name;
		$this->row_name = $row_name;
		$this->avoid_duplicates = $avoid_duplicates;
		$this->encode_entities = $encode_entities;

		$this->written_rows = Array();

		if ( preg_match( '~\.xml$~i', $file_name ) )
		{
			$fn_parts = explode( '.', $file_name );
			array_pop( $fn_parts );
			$this->file_name = implode( '.', $fn_parts );
		}
		else
		{
			$this->file_name = $file_name;
		}

		$this->open( $this->file_name.'.xml' );
	}

	/**
	 * ffnet die Ausgabedatei zum Schreiben.
	 *
	 * @param String $file_name Der Name der Ausgabedatei
	 */
	function open( $file_name )
	{
		$this->file_handle = fopen( $file_name, 'w+' );
		fputs( $this->file_handle, '<?xml version="1.0" encoding="iso-8859-1"?>'."\n" );
		fputs( $this->file_handle, '<'.$this->root_name.'>'."\n" );
	}

	/**
	 * Setzt die Variable des Duplikatvermeidens.
	 *
	 * @param boolean $state Neuer Zustand
	 */
	function setAvoidDuplicates( $state )
	{
		$this->avoid_duplicates = $state;
	}

	/**
	 * Setzt die neue maximale Gre der Ausgabedatei.
	 *
	 * Angaben von Kilobyte- oder Megabyte-Angaben wie 512kb oder 10mb sind zulssig, drfen aber
	 * nicht gemischt auftreten.
	 *
	 * @param boolean $max_size Neue Maximale Gre der Ausgabedatei (0 fr ausschalten)
	 */
	function setMaxSize( $max_size )
	{
		if ( strpos( $max_size, 'kb' ) !== false )
		{
			$max_size = str_replace( 'kb', '', $max_size );
			$max_size *= 1024;
		}
		elseif ( strpos( $max_size, 'kb' ) !== false )
		{
			$max_size = str_replace( 'mb', '', $max_size );
			$max_size *= 1024*1024;
		}

		$this->max_size = $max_size;
	}

	/**
	 * Speichert eine als Array bergebene Zeile in dem XML-File
	 *
	 * @param Array $rowData Die Daten der Zeile
	 */
	function saveRow( $rowData )
	{
		if ( $this->avoid_duplicates )
		{
			$data = '';
			foreach ( $rowData as $key => $value )
			{
				$data .= $key.$value;
			}
			$hash = md5( $data );
			if ( !empty( $this->written_rows[ $hash ] ) )
			{
				return;
			}
			$this->written_rows[ $hash ] = true;
		}

		$this_row = '  <'.$this->row_name.'>'."\n";
		foreach ( $rowData as $key => $value )
		{
			if ( $this->encode_entities )
			{
				$value = $this->xmlentities( $value );
			}
			$this_row .= '    <'.$key.'>'.$value.'</'.$key.'>'."\n";
		}
		$this_row .= '  </'.$this->row_name.'>'."\n";

		if ( $this->max_size > 0 )
		{
			if ( $this->part_size + strlen( $this_row ) > $this->max_size and $this->part_size > 0 )
			{
				$this->close();
				if ( $this->written_parts++ == 0 )
				{
					rename( $this->file_name.'.xml', $this->file_name.'.part0.xml' );
				}
				$this->open( $this->file_name.'.part'.$this->written_parts.'.xml' );
				$this->part_size = 0;
			}
			$this->part_size += strlen( $this_row );
		}
		fputs( $this->file_handle, $this_row );
		fflush( $this->file_handle );
	}

	/**
	 * Speichert mehrere als Array bergebene Zeilen in dem XML-File
	 *
	 * @param Array $rowsData Die Zeilen als Array
	 * @see saveRow
	 */
	function saveRows( $rowsData )
	{
		foreach ( $rowsData as $row )
		{
			$this->saveRow( $row );
		}
	}

	/**
	 * Ersetzt bestimmte Zeichen durch ihre XML-Entitten
	 *
	 * @param String $string Die Zeichenkette, in der die Entitten ersetzt werden sollen
	 * @return String Der bearbeitete String
	 */
	function xmlentities( $string )
	{
		return str_replace( Array( '&', '<', '>' ), Array( '&amp;', '&lt;', '&gt;' ), $string );
	}

	/**
	 * Schliesst das XML-File
	 *
	 */
	function close()
	{
		fputs( $this->file_handle, '</'.$this->root_name.'>'."\n" );
		fclose( $this->file_handle );
	}
}
?>